@@ -38,13 +38,54 @@
 #define DIRCHR '/'
 #endif
 
+/**
+ * pointer_switch_addresses:
+ * a: #gpointer;
+ * b: #gpointer
+ *
+ * after this call, a will contain the address previously in a
+ * and b will contain the address previously in b
+ *
+ * Return value: void
+ */
+#ifdef __GNUC__
+__inline__ 
+#endif
+void pointer_switch_addresses(gpointer *a, gpointer *b) {
+	gpointer c;
+	DEBUG_MSG("pointer_switch_addresses, before, a=%p, b=%p\n",a,b);
+	c = *a;
+	*a = *b;
+	*b = c;
+	DEBUG_MSG("pointer_switch_addresses, after, a=%p, b=%p\n",a,b);
+}
+
+/**
+ * list_switch_order:
+ * @first: a #GList * item
+ * @second: a #GList * item
+ * 
+ * this function will switch place of these two list items
+ * actually not the items themselves, but the data they are 
+ * pointer to is switched
+ * 
+ * Return value: void
+ **/
 void list_switch_order(GList *first, GList *second) {
 	gpointer tmp;
 	tmp = first->data;
 	first->data = second->data;
 	second->data = tmp;
 }
-
+/**
+ * file_copy:
+ * @source: a #gchar * containing the source filename
+ * @dest: a #gchar * containing the destination filename
+ * 
+ * copies the contents of the file source to dest
+ * 
+ * Return value: gboolean, TRUE if the function succeeds
+ **/
 gboolean file_copy(gchar *source, gchar *dest) {
 #ifdef DEVELOPMENT
 	g_assert(source);
@@ -69,7 +110,16 @@ gboolean file_copy(gchar *source, gchar *dest) {
 	fclose(out);
 	return TRUE;
 }
-
+/**
+ * append_string_to_file:
+ * @filename: a #gchar * containing the destination filename
+ * @string: a #gchar * containing the string to append
+ * 
+ * opens the file filename in append mode, and appends the string
+ * no newline or anything else is appended, just the string
+ * 
+ * Return value: gboolean, TRUE if the function succeeds
+ **/
 gboolean append_string_to_file(gchar *filename, gchar *string) {
 	FILE *out;
 
@@ -82,7 +132,16 @@ gboolean append_string_to_file(gchar *filename, gchar *string) {
 	fclose(out);
 	return TRUE;
 }
-
+/**
+ * table_convert_char2int:
+ * @table: a #tconvert_table * with strings and integers
+ * @my_char: a #gchar * containing the string to convert
+ * 
+ * this function can be used to translate a string from some set (in table)
+ * to an integer
+ * 
+ * Return value: gint, found in table, or -1 if not found
+ **/
 gint table_convert_char2int(Tconvert_table *table, gchar *my_char) {
 	Tconvert_table *entry;
 	entry = table;
@@ -94,9 +153,18 @@ gint table_convert_char2int(Tconvert_table *table, gchar *my_char) {
 	}
 	return -1;
 }
-/* 
- * WARNING: does NOT malloc any memory, do not free!!
- */
+/**
+ * table_convert_int2char:
+ * @table: a #tconvert_table * with strings and integers
+ * @my_int: a #gint containing the integer to convert
+ * 
+ * this function can be used to translate an integer from some set (in table)
+ * to a string
+ * WARNING: This function will return a pointer into table, it will 
+ * NOT allocate new memory
+ * 
+ * Return value: gchar * found in table, else NULL
+ **/
 gchar *table_convert_int2char(Tconvert_table *table, gint my_int) {
 	Tconvert_table *entry;
 	entry = table;
@@ -108,7 +176,17 @@ gchar *table_convert_int2char(Tconvert_table *table, gint my_int) {
 	}
 	return NULL;
 }
-
+/**
+ * replace_string_printflike:
+ * @string: a formatstring #gchar * to convert
+ * @table: a #Tconvert_table * to use for conversion
+ * 
+ * this function can convert a format string with %0, %1 etc. 
+ * into the final string, where each %number entry is replaced 
+ * with the string found in table
+ * 
+ * Return value: a newly allocated gchar * with the resulting string
+ **/
 gchar *replace_string_printflike(const gchar *string, Tconvert_table *table) {
 	gchar *p, *prev, *stringdup;
 	gchar *tmp, *dest = g_strdup("");
@@ -156,20 +234,40 @@ typedef struct {
 } Tutf8_offset_cache;
 
 static Tutf8_offset_cache utf8_offset_cache;
+
+/**
+ * utf8_offset_cache_reset:
+ * 
+ * this function will reset the utf8 offset cache used by 
+ * utf8_byteoffset_to_charsoffset_cached()
+ *
+ * normally this is done automatically if utf8_byteoffset_to_charsoffset_cached() 
+ * is called with a new buffer. But if you ever call that function for 
+ * the same buffer but the buffer is changed in the meantime you have 
+ * to reset it manually using utf8_offset_cache_reset()
+ * 
+ * Return value: void
+ **/
 #ifdef __GNUC__
 __inline__ 
 #endif
 void utf8_offset_cache_reset() {
-	/* set all to 0, and set the last_buf to NULL */
-/*	gint i;
-	for (i=0;i<UTF8_OFFSET_CACHE_SIZE;i++) {
-		utf8_offset_cache.last_byteoffset[i] = 0;
-		utf8_offset_cache.last_charoffset[i] = 0;
-	}
-	utf8_offset_cache.last_buf = NULL;*/
 	memset(&utf8_offset_cache, 0, sizeof(Tutf8_offset_cache));
 }
-
+/**
+ * utf8_byteoffset_to_charsoffset_cached:
+ * @string: the gchar * you want to count
+ * @byteoffset: glong with the byteoffset you want the charoffset for
+ * 
+ * this function calculates the UTF-8 character offset in a string for
+ * a given byte offset
+ * It uses caching to speedup multiple calls for the same buffer, the cache
+ * is emptied if you change to another buffer. If you use the same buffer but 
+ * change it inbetween calls, you have to reset it yourself using
+ * the utf8_offset_cache_reset() function
+ * 
+ * Return value: guint with character offset
+ **/
 guint utf8_byteoffset_to_charsoffset_cached(gchar *string, glong byteoffset) {
 	guint retval;
 	gint i = UTF8_OFFSET_CACHE_SIZE-1;
@@ -178,6 +276,9 @@ guint utf8_byteoffset_to_charsoffset_cached(gchar *string, glong byteoffset) {
 		utf8_offset_cache_reset();
 		utf8_offset_cache.last_buf = string;
 	}
+#ifdef DEBUG
+	DEBUG_MSG("utf8_byteoffset_to_charsoffset_cached, string %p has strlen %d\n", string, strlen(string));
+#endif
 
 	while (i > 0 && utf8_offset_cache.last_byteoffset[i] > byteoffset) {
 		i--;
@@ -190,11 +291,6 @@ guint utf8_byteoffset_to_charsoffset_cached(gchar *string, glong byteoffset) {
 	}
 	if (i == (UTF8_OFFSET_CACHE_SIZE-1)) {
 		/* add this new calculation to the cache */
-/*		gint j;
-		for (j=0;j<(UTF8_OFFSET_CACHE_SIZE-1);j++) {
-			utf8_offset_cache.last_byteoffset[j] = utf8_offset_cache.last_byteoffset[j+1];
-			utf8_offset_cache.last_charoffset[j] = utf8_offset_cache.last_charoffset[j+1];	
-		}*/
 		/* this is a nasty trick to move all guint entries one back in the array, so we can add the new one */
 		memmove(&utf8_offset_cache.last_byteoffset[0], &utf8_offset_cache.last_byteoffset[1], (UTF8_OFFSET_CACHE_SIZE+UTF8_OFFSET_CACHE_SIZE-1)*sizeof(guint));
 
@@ -204,14 +300,16 @@ guint utf8_byteoffset_to_charsoffset_cached(gchar *string, glong byteoffset) {
 	return retval;
 }
 
-/**************************************************/
-/**************************************************/
-
-
-
-/* static guint countchars(gchar *string, gchar *chars)
- * counts the amount of times the chars in chars are present in string */
-static guint countchars(gchar *string, gchar *chars) {
+/**
+ * countchars:
+ * @string: a gchar * to count the chars in
+ * @chars: a gchar * with the characters you are interested in
+ *
+ * this function will count every character in string that is also in chars
+ * 
+ * Return value: guint with the number of characters found
+ **/
+guint countchars(gchar *string, gchar *chars) {
 	guint count=0;
 	gchar *newstr = strpbrk(string, chars);
 	while(newstr) {
@@ -221,8 +319,16 @@ static guint countchars(gchar *string, gchar *chars) {
 	return count;
 }
 
-/* gchar *escapestring(gchar *original, gchar delimiter)
- * returns a newly allocated string which is backslash escaped */
+/**
+ * escapestring:
+ * @original: a gchar * to escape
+ * @delimiter: a gchar that needs escaping, use '\0' if you don't need one
+ *
+ * this function will backslash escape \n, \t, and \ characters, and if 
+ * there is a delimiter it will also be escaped
+ * 
+ * Return value: a newly allocated gchar * that is escaped
+ **/
 gchar *escapestring(gchar *original, gchar delimiter)
 {
 	gchar *tmp, *newstring, *escapedchars;
@@ -269,8 +375,16 @@ gchar *escapestring(gchar *original, gchar delimiter)
 	return newstring;
 }
 
-/* gchar *unescapestring(gchar *original)
- * returns a newly allocated string which is backslash un-escaped */
+/**
+ * escapestring:
+ * @original: a gchar * to unescape
+ *
+ * this function will backslash unescape \n, \t, and \\ sequences to 
+ * their characters, and if there is any other escaped character 
+ * it will be replaced without the backslash
+ * 
+ * Return value: a newly allocated gchar * that is unescaped
+ **/
 gchar *unescapestring(gchar *original)
 {
 	gchar *tmp1, *tmp2, *newstring;
@@ -318,9 +432,15 @@ gchar *unescapestring(gchar *original)
 	return newstring;
 }
 
-
-/* gboolean change_dir(gchar * filename)
- * changes the current working directory to the directory of filename */
+/**
+ * change_dir:
+ * @filename: a gchar * with the path (may be a filename!!) to change to
+ *
+ * this function will get the directory name from the filename
+ * and chdir() to that directory
+ * 
+ * Return value: gboolean, returning what chdir() returned
+ **/
 gboolean change_dir(gchar * filename)
 {
 	gchar *tmpstring;
@@ -328,7 +448,7 @@ gboolean change_dir(gchar * filename)
 
 	if (!filename)
 		return FALSE;
-	tmpstring = g_dirname(filename);
+	tmpstring = g_path_get_dirname(filename);
 	DEBUG_MSG("change_dir, filename = %s, tmpstring = %s in %p\n", filename, tmpstring, tmpstring);
 	returncode = chdir(tmpstring) == 0 ? TRUE : FALSE;
 	g_free(tmpstring);
@@ -336,9 +456,16 @@ gboolean change_dir(gchar * filename)
 	return returncode;
 }
 
-/* gchar *strip_any_whitespace(gchar *string)
+/**
+ * strip_any_whitespace:
+ * @string: a gchar * to strip
+ *
  * strips any double chars defined by isspace() from the string, 
- * only single spaces are returned */
+ * only single spaces are returned
+ * the same string is returned, no memory is allocated in this function
+ * 
+ * Return value: the same gchar * as passed to the function
+ **/
 gchar *strip_any_whitespace(gchar *string) {
 	gint count=0, len;
 
@@ -368,13 +495,16 @@ gchar *strip_any_whitespace(gchar *string) {
 	DEBUG_MSG("strip_any_whitespace, returning string='%s'\n", string);
 	return string;
 }
-
-
-/* gchar *trunc_on_char(gchar * string, gchar which_char)
- *      string - text string
- *      which_char      - termination character
+/**
+ * trunc_on_char:
+ * @string: a #gchar * to truncate
+ * @which_char: a #gchar with the char to truncate on
+ *
  * Returns a pointer to the same string which is truncated at the first
- * occurence of which_char */
+ * occurence of which_char
+ * 
+ * Return value: the same gchar * as passed to the function
+ **/
 gchar *trunc_on_char(gchar * string, gchar which_char)
 {
 	gchar *tmpchar = string;
@@ -387,13 +517,12 @@ gchar *trunc_on_char(gchar * string, gchar which_char)
 	}
 	return string;
 }
-
 /* gchar *strip_common_path(char *image_fn, char *html_fn)
  * returns a newly allocated string containing the the to_filename
  * but all the common path with from_filename is removed 
  *
- * IS THIS IN USE ?? */ 
-gchar *strip_common_path(char *to_filename, char *from_filename)
+ * IS THIS IN USE ?? OBVIOUSLY NOT BECAUSE I CAN REMOVE IT */ 
+/*gchar *strip_common_path(char *to_filename, char *from_filename)
 {
 	gchar *tempstr;
 	int count, count2, dir_length;
@@ -410,26 +539,25 @@ gchar *strip_common_path(char *to_filename, char *from_filename)
 			break;
 		}
 	}
-
-	/* contributed by Tony Kemp <tony.kemp@studentmail.newcastle.edu.au> */
 	while (to_filename[count - 1] != DIRCHR)
 		count--;
-
-
 	DEBUG_MSG("strip_common_path, equal count = %d\n", count);
 	count2 = strlen(to_filename);
 	tempstr = g_malloc(count2 - count + 2);
 	memcpy(tempstr, &to_filename[count], count2 - count + 2);
 	DEBUG_MSG("strip_common_path, tempstr= %s, should be %d long\n", tempstr, count2 - count);
 	return tempstr;
-	/* the function that calls this one must use gfree() somewhere!! */
-}
+} */
 
-
-/* gchar *most_efficient_filename(gchar *filename)
- *      filename: the string which will be modified to remove dir/../
- * Returns the same string, but modified , 
- * not wanted dir/../ combinations are removed */
+/**
+ * most_efficient_filename:
+ * @filename: a gchar * with a possibly inefficient filename like /hello/../tmp/../myfile
+ *
+ * tries to eliminate any dir/../ combinations in filename
+ * this function could do evern better, it should also remove /./ entries
+ * 
+ * Return value: the same gchar * as passed to the function
+ **/
 gchar *most_efficient_filename(gchar *filename) {
 	gint i,j, len;
 
@@ -452,11 +580,21 @@ gchar *most_efficient_filename(gchar *filename) {
 	return filename;
 }
 
-
-/* gchar *create_relative_link_to(gchar * current_filepath, gchar * link_to_filepath)
- * 	current_filepath: the file to link FROM, a full path
- * 	link_to_filepath: the file to link TO, a full path
- * Returns a newly allocated string containing a relative path */
+/**
+ * create_relative_link_to:
+ * @current_filepath: a #gchar * with the current filename
+ * @link_to_filepath: a #gchar * with a file to link to
+ *
+ * creates a newly allocated relative link from current_filepath 
+ * to link_to_filepath
+ *
+ * if current_filepath == NULL it returns the most efficient filepath 
+ * for link_to_filepath
+ *
+ * if link_to_filepath == NULL it will return NULL
+ *
+ * Return value: a newly allocated gchar * with the relative link
+ **/
 gchar *create_relative_link_to(gchar * current_filepath, gchar * link_to_filepath)
 {
 	gchar *returnstring = NULL;
@@ -474,10 +612,9 @@ gchar *create_relative_link_to(gchar * current_filepath, gchar * link_to_filepat
 	}
 	eff_current_filepath = most_efficient_filename(g_strdup(current_filepath));
 	eff_link_to_filepath = most_efficient_filename(g_strdup(link_to_filepath));
-
-	/* get the size of the directory    of the current_filename */
-
-	
+	DEBUG_MSG("eff_current: '%s'\n",eff_current_filepath);
+	DEBUG_MSG("eff_link_to: '%s'\n",eff_link_to_filepath);
+	/* get the size of the directory of the current_filename */
 	current_filename_length = strlen(strrchr(eff_current_filepath, DIRCHR))-1;
 	link_to_filename_length = strlen(strrchr(eff_link_to_filepath, DIRCHR))-1;
 	DEBUG_MSG("create_relative_link_to, filenames: current: %d, link_to:%d\n", current_filename_length,link_to_filename_length); 
@@ -537,14 +674,20 @@ gchar *create_relative_link_to(gchar * current_filepath, gchar * link_to_filepat
 	return returnstring;
 }
 
-
-/* gchar *create_full_path(gchar * filename, gchar *basedir)
- * 	filename: the relative filename
- * 	basedir: a dir or file in the dir used as basedir for the filename
- * 		if this is NULL the current working directory is used
- * Returns a newly allocated string containing a absolute path */
-gchar *create_full_path(gchar * filename, gchar *basedir)
-{
+/**
+ * create_full_path:
+ * @filename: a gchar * with the (relative or not) filename
+ * @basedir: a gchar * with a basedir or NULL for current dir
+ *
+ * if filename is already absolute, it returns it
+ * else it will use basedir if available, else the current dir
+ * to add to the filename to form the full path
+ *
+ * it does use most_efficient_filename() to remote unwanted dir/../ entries
+ *
+ * Return value: a newly allocated gchar * with the full path
+ **/
+gchar *create_full_path(gchar * filename, gchar *basedir) {
 	gchar *absolute_filename;
 	gchar *tmpcdir;
 
@@ -566,13 +709,16 @@ gchar *create_full_path(gchar * filename, gchar *basedir)
 	return absolute_filename;
 }
 
-
-
-/* gchar *ending_slash(const gchar *dirname)
- * returns a newly allocated string which is 
- * guaranteed to end on a slash (/) */
+/**
+ * ending_slash:
+ * @dirname: a #const gchar * with a diretory name
+ *
+ * makes sure the last character of the newly allocated 
+ * string it returns is a '/'
+ *
+ * Return value: a newly allocated gchar * dirname that does end on a '/'
+ **/
 gchar *ending_slash(const gchar *dirname) {
-	/* result should be free-ed !!! */
 	if (!dirname) {
 		return g_strdup("");
 	}
@@ -583,21 +729,36 @@ gchar *ending_slash(const gchar *dirname) {
 		return g_strconcat(dirname, DIRSTR, NULL);
 	}
 }
-
-/* gint file_is_dir(gchar * filename)
- * returns 1 if the file pointed to by filename is a dir
- * else returns 0 */
-#ifdef __GNUC__
-__inline__ 
-#endif
-gint file_is_dir(gchar * filename) {
-	return g_file_test(filename, G_FILE_TEST_IS_DIR);
+/**
+ * path_get_dirname_with_ending_slash:
+ * @filename: a #const gchar * with a file path
+ *
+ * returns a newly allocated string, containing everything up to 
+ * the last '/' character, including that character itself.
+ *
+ * if no '/' character is found it returns NULL
+ *
+ * Return value: a newly allocated gchar * dirname that does end on a '/', or NULL on failure
+ **/
+gchar *path_get_dirname_with_ending_slash(const gchar *filename) {
+	gchar *tmp = strrchr(filename, DIRCHR);
+	if (tmp) {
+		return g_strndup(filename, (tmp - filename + 1));
+	} else {
+		return NULL;
+	}
 }
-
-/* gint file_exists_and_readable(gchar * filename)
- * returns 1 if the file pointed to by filename does exist
- * and is readable for the current process */
-gint file_exists_and_readable(const gchar * filename)
+/**
+ * file_exists_and_readable:
+ * @filename: a #const gchar * with a file path
+ *
+ * tests if the file exists, using stat(), and is readable, the last
+ * check is not reliable, it does not check all the groups you are
+ * in, so change this function before you rely on that check!
+ *
+ * Return value: gboolean, TRUE if readable, else FALSE
+ **/
+gboolean file_exists_and_readable(const gchar * filename)
 {
 	struct stat naamstat;
 
@@ -607,23 +768,32 @@ gint file_exists_and_readable(const gchar * filename)
 
 	if (!filename || strlen(filename) < 2) {
 		DEBUG_MSG("file_exists_and_readable, strlen(filename) < 2 or no filename!!!!\n");
-		return 0;
+		return FALSE;
 	}
 	DEBUG_MSG("file_exists_and_readable, filename(%p)=\"%s\", strlen(filename)=%d\n", filename, filename, strlen(filename));
 #ifndef WIN32
 	if ((stat(filename, &naamstat) == -1) && (errno == ENOENT)) {
 /*              if(naamstat.st_mode & S_IREAD) { */
-		return (0);
+		return FALSE;
 /*              }
    return(0); */
 	} else {
-		return (1);
+		return TRUE;
 	}
 #else
-	return (1);
+	return TRUE;
 #endif
 }
-
+/**
+ * return_first_existing_filename:
+ * @filename: a #const gchar * with a filename
+ * @...: more filenames
+ *
+ * you can pass multiple filenames to this function, and it will return
+ * the first filename that really exists according to file_exists_and_readable()
+ *
+ * Return value: gchar * with the first filename found
+ **/
 gchar *return_first_existing_filename(const gchar *filename, ...) {
 	va_list args;
 	gchar *retval=NULL;
@@ -631,7 +801,7 @@ gchar *return_first_existing_filename(const gchar *filename, ...) {
 	va_start(args, filename);
 	while (filename) {
 		if (file_exists_and_readable(filename)) {
-			retval = filename;
+			retval = g_strdup(filename);
 			break;
 		}
 		filename = va_arg(args, gchar*);
@@ -640,11 +810,16 @@ gchar *return_first_existing_filename(const gchar *filename, ...) {
 	return retval;
 }
 
-/*  gboolean filename_test_extensions(gchar **extensions, gchar *filename)
- *  give a NULL terminated array of strings with all extensions
- *  returns TRUE if the filename has one of those extensions
- *  returns FALSE if not
- */
+/**
+ * filename_test_extensions:
+ * @extensions: a #gchar ** NULL terminated arrau of strings
+ * @filename: a #const gchar * with a filename
+ *
+ * tests if the filename matches one of the extensions passed in the NULL terminated array
+ * of strings
+ *
+ * Return value: gboolean, TRUE if the file has one of the extensions in the array
+ **/
 gboolean filename_test_extensions(gchar **extensions, gchar *filename) {
 	if (!extensions) {
 		return FALSE;
@@ -658,13 +833,16 @@ gboolean filename_test_extensions(gchar **extensions, gchar *filename) {
 	return FALSE;
 }
 
-/*
- * gchar *bf_str_repeat(const gchar * str, gint number_of)
- *      str - pointer to string
- *      number_of - number copies of string
- * returns:     pointer to new allocated string
- *      calculate, allocate buffer and copies number_of times str to new buffer
- */
+/**
+ * bf_str_repeat:
+ * @str: a #const gchar * 
+ * @number_of: a #gint
+ *
+ * returns a newly allocated string, 
+ * containing str repeated number_of times
+ *
+ * Return value: the newly allocated #gchar *
+ **/
 gchar *bf_str_repeat(const gchar * str, gint number_of) {
 	gchar *retstr;
 	gint len = strlen(str) * number_of;
@@ -677,10 +855,16 @@ gchar *bf_str_repeat(const gchar * str, gint number_of) {
 	return retstr;
 }
 
-/* gint get_int_from_string(gchar *string)
+/**
+ * get_int_from_string:
+ * @string: a #const gchar * 
+ *
  * tries to find a positive integer value in the string and returns it
- * returns -1 on failure
- */
+ * the string does not have to start or end with the integer
+ * it returns -1 if no integer was found somewhere in the string
+ *
+ * Return value: the found #gint, -1 on failure
+ **/
 gint get_int_from_string(gchar *string) {
 	if (string) {
 		gint faktor = 1, result=-1;
@@ -704,15 +888,20 @@ gint get_int_from_string(gchar *string) {
 	}
 	return -1;
 }
-/* gchar *create_secure_dir_return_filename()
+
+/**
+ * create_secure_dir_return_filename:
+ *
  * this function uses mkdir(name) to create a dir with permissions rwx------
  * mkdir will fail if name already exists or is a symlink or something
  * the name is chosen by tempnam() so the chance that mkdir() fails in
  * a normal situation is minimal, it almost must be a hacking attempt
  *
  * the filename generated can safely be used for output of an external 
- * script
- */
+ * script because the dir has rwx------ permissions
+ *
+ * Return value: a newly allocated #gchar * containing a temporary filename in a secure dir
+ **/
 gchar *create_secure_dir_return_filename() {
 	gchar *name, *name2;
 	DEBUG_MSG("create_secure_dir_return_filename,g_get_tmp_dir()=%s\n", g_get_tmp_dir());
@@ -735,9 +924,18 @@ gchar *create_secure_dir_return_filename() {
 	g_free(name);
 	return name2;
 }
-
+/**
+ * remove_secure_dir_and_filename:
+ * @filename: the #gchar * filename to remove
+ *
+ * this function will remove a the filename created 
+ * by create_secure_dir_return_filename(), and the safe
+ * directory the file was created in
+ *
+ * Return value: void
+ **/
 void remove_secure_dir_and_filename(gchar *filename) {
-	gchar *dirname = g_dirname(filename);
+	gchar *dirname = g_path_get_dirname(filename);
 	unlink(filename);
 	rmdir(dirname);
 	g_free(dirname);
@@ -754,3 +952,66 @@ void remove_secure_dir_and_filename(gchar *filename) {
 	}
 	return buf;
 }*/
+
+/**
+ * wordcount:
+ * @text: A #gchar* to examine.
+ * @chars: #guint*, will contain no. chars in text.
+ * @lines: #guint*, will contain no. lines in text.
+ * @words: #guint*, will contain no. words in text.
+ *
+ * Returns number of characters, lines and words in the supplied #gchar*.
+ * Handles UTF-8 correctly. Input must be properly encoded UTF-8.
+ * Words are defined as any characters grouped, separated with spaces.
+ * I.e., this is a word: "12a42a,.". This is two words: ". ."
+ *
+ * This function contains a switch() with inspiration from the GNU wc utility.
+ * Rewritten for glib UTF-8 handling by Christian Tellefsen, chris@tellefsen.net.
+ *
+ * Note that gchar == char, so that gives us no problems here.
+ *
+ * Return value: void
+ **/
+void wordcount(gchar *text, guint *chars, guint *lines, guint *words)
+{
+	guint in_word = 0;
+	gunichar utext;
+	
+	if(!text) return; /* politely refuse to operate on NULL .. */
+		
+	*chars = *words = *lines = 0;
+	while (*text != '\0') {
+		(*chars)++;
+		
+		switch (*text) {
+			case '\n':
+				(*lines)++;
+				/* Fall through. */
+			case '\r':
+			case '\f':
+			case '\t':
+			case ' ':
+			case '\v':
+				mb_word_separator:
+				if (in_word) {
+					in_word = 0;
+					(*words)++;
+				}
+				break;
+			default:
+				utext = g_utf8_get_char_validated(text, 2); /* This might be an utf-8 char..*/
+				if (g_unichar_isspace (utext)) /* Unicode encoded space? */
+					goto mb_word_separator;
+				if (g_unichar_isgraph (utext)) /* Is this something printable? */
+					in_word = 1;
+				break;
+		} /* switch */
+		
+		text = g_utf8_next_char(text); /* Even if the current char is 2 bytes, this will iterate correctly. */
+	}
+	
+	/* Capture last word, if there's no whitespace at the end of the file. */
+	if(in_word) (*words)++;
+	/* We start counting line numbers from 1 */
+	if(*chars > 0) (*lines)++;
+}
