@@ -26,6 +26,7 @@ typedef struct {
 	gchar *my_char;
 } Tconvert_table;
 
+void pointer_switch_addresses(gpointer *a, gpointer *b);
 void list_switch_order(GList *first, GList *second);
 gboolean file_copy(gchar *source, gchar *dest);
 gboolean append_string_to_file(gchar *filename, gchar *string);
@@ -47,13 +48,14 @@ gchar *most_efficient_filename(gchar *filename);
 gchar *create_relative_link_to(gchar * current_filepath, gchar * link_to_filepath);
 gchar *create_full_path(gchar * filename, gchar *basedir);
 gchar *ending_slash(const gchar *dirname);
-gint file_is_dir(gchar * filename);
-gint file_exists_and_readable(const gchar * filename);
+gchar *path_get_dirname_with_ending_slash(const gchar *filename);
+gboolean file_exists_and_readable(const gchar * filename);
 gchar *return_first_existing_filename(const gchar * filename, ...);
 gboolean filename_test_extensions(gchar **extensions, gchar *filename);
 gchar *bf_str_repeat(const gchar * str, gint number_of);
 gint get_int_from_string(gchar *string);
 gchar *create_secure_dir_return_filename();
 void remove_secure_dir_and_filename(gchar *filename);
-gchar *buf_replace_char(gchar *buf, gint len, gchar srcchar, gchar destchar);
+/*gchar *buf_replace_char(gchar *buf, gint len, gchar srcchar, gchar destchar);*/
+void wordcount(gchar *text, guint *chars, guint *lines, guint *words);
 #endif /* __BF_LIB_H_ */
