@@ -92,6 +92,8 @@ typedef struct {
 	GList *highlightlist;
 	GtkWidget *menuitem; /* the menuitem to choose this for highlighting */
 	gulong menuitem_activate_id;
+	gboolean editable; /* this a type that can be edited by Bluefish */
+	gchar *content_regex; /* a regex pattern to test the filetype using the content */
 } Tfiletype;
 
 /*******************/
@@ -121,6 +123,7 @@ typedef struct {
 	gboolean highlightstate; /* does this document use highlighting ? */
 	gboolean wrapstate; /* does this document use wrap?*/
 	gboolean linenumberstate; /* does this document use linenumbers? */
+	gboolean overwrite_mode; /* is document in overwrite mode */
 } Tdocument;
 
 typedef struct {
@@ -136,13 +139,18 @@ typedef struct {
 	gchar *filebrowser_dir_icon;
 	gchar *editor_font_string;		/* editor font */
 	gint editor_tab_width;	/* editor tabwidth */
+	gint editor_indent_wspaces; /* indent with spaces, not tabs */
 	gchar *tab_font_string;		/* notebook tabs font */
 	GList *browsers; /* browsers array */
 	GList *external_commands;	/* external commands array */
+	GList *cust_menu; 		/* entries in the custom menu */
+	GList *cmenu_insert; /* custom menu inserts */
+	GList *cmenu_replace; /* custom menu replaces */
 	GList *quickbar_items; /* items in the quickbar toolbar */
 	gint highlight_num_lines_count; /* number of lines to highlight in continous highlighting */	
 	gint defaulthighlight;		/* highlight documents by default */
 	GList *filetypes; /* filetypes for highlighting and filtering */
+	gint numcharsforfiletype; /* maximum number of characters in the file to use to find the filetype */
 	GList *filefilters; /* filebrowser.c filtering */
 	gchar *last_filefilter;	/* last filelist filter type */
 	GList *highlight_patterns; /* the highlight patterns */
@@ -155,7 +163,8 @@ typedef struct {
 	gint max_dir_history;	/* length of directory history */
 	gint backup_file; 			/* wheather to use a backup file */
 	gchar *backup_filestring;  /* the string to append to the backup file */
-	gchar *backup_abort_style; /* if the backup fails, continue 'save', 'abort' save, or 'ask' user */
+	gint backup_abort_action; /* if the backup fails, continue save, abort save, or ask the user */
+	gint backup_cleanuponclose; /* remove the backupfile after close ? */
 	gchar *image_thumbnailstring;	/* string to append to thumbnail filenames */
 	gchar *image_thumbnailtype;	/* fileformat to use for thumbnails, "jpeg" or "png" can be handled by gdkpixbuf*/
 	gint image_thumbnail_refresh_quality; /* 1=GDK_INTERP_BILINEAR, 0=GDK_INTERP_NEAREST*/
@@ -174,15 +183,22 @@ typedef struct {
 	gint ext_browsers_in_submenu;
 	gint ext_commands_in_submenu;
 	gint ext_outputbox_in_submenu;
+	GList *reference_files; /* all reference files */
+	gint fref_ldoubleclick_action; /* left doubleclick in the function reference */
+	gint fref_info_type; /* type of info shown in a small function reference window */
+	gint document_tabposition;
+	gint leftpanel_tabposition;
 #ifdef HAVE_LIBASPELL
 	gchar *spell_default_lang;
 #endif /* HAVE_LIBASPELL */
 	gint default_advanced_snr;
 	gint cont_highlight_full; 	/* if you want to highlight the full text or just the line */
+	gint lasttime_cust_menu; /* the last time the defaultfile was checked for new entries */
+	gint lasttime_highlighting; /* see above */
+	gint lasttime_filetypes; /* see above */
+	gint lasttime_encodings; /* see above */
 	/* not yet in use */
 	gchar *image_editor_cline; 	/* image editor commandline */
-	gchar *cfg_weblint_cline;	/* weblint command line */
-	gchar *cfg_tab_pos;			/* notebook tabs positioning */
 	gint full_p;				/* use </p> */
 	gint full_li;				/* use </li> */
 	gint allow_css;				/* CSS allowed */
@@ -204,7 +220,6 @@ typedef struct {
 	gint link_management; 	/* perform link management */
 	gint cont_highlight_update;	/* update the syntax highlighting continuous */
 	gchar *html_ver;
-	GList *cust_menu; 		/* entries in the custom menu */
 	/* key conversion */
 	gint conv_ctrl_enter;		/* convert control-enter key press */
 	gchar *ctrl_enter_text;		/* inserted text */
@@ -225,10 +240,14 @@ typedef struct {
 	gint last_notebook_page; /* a check to see if the notebook changed to a new page */
 	gulong notebook_switch_signal;
 	GtkWidget *notebook;
+	GtkWidget *notebook_fake;
+	GtkWidget *notebook_box; /* Container for notebook and notebook_fake */
 	GtkWidget *middlebox; /* we need this to show/hide the filebrowser */
 	GtkWidget *hpane; /* we need this to show/hide the filebrowser */
 	GtkWidget *statusbar;
-	GtkWidget *statuslabel; /* where we have the line number */
+	GtkWidget *statusbar_lncol; /* where we have the line number */
+	GtkWidget *statusbar_insovr; /* insert/overwrite indicator */
+	GtkWidget *statusbar_editmode; /* editor mode and doc encoding */
 	GList *recent_directories; /* a stringlist with the most recently used directories */
 } Tmain;
 
