@@ -21,10 +21,21 @@
 #ifndef __DOCUMENT_H_
 #define __DOCUMENT_H_
 
+enum {
+	DOCUMENT_BACKUP_ABORT_SAVE,
+	DOCUMENT_BACKUP_ABORT_ABORT,
+	DOCUMENT_BACKUP_ABORT_ASK
+};
+
 gint documentlist_return_index_from_filename(gchar *filename);
+Tdocument *documentlist_return_document_from_index(gint index);
 
 void doc_update_highlighting(GtkWidget *wid, gpointer data);
 void document_set_wrap(Tdocument *doc, gint wraptype);
+gboolean doc_set_filetype(Tdocument *doc, Tfiletype *ft);
+Tfiletype *get_filetype_by_name(gchar * name);
+Tfiletype *get_filetype_by_filename_and_content(gchar *filename, gchar *buf);
+void doc_reset_filetype(Tdocument * doc, gchar * newfilename, gchar *buf);
 void doc_set_font(Tdocument *doc, gchar *fontstring);
 void doc_set_tabsize(Tdocument *doc, gint tabsize);
 void gui_change_tabsize(gpointer callback_data,guint action,GtkWidget *widget);
@@ -42,6 +53,8 @@ void doc_select_region(Tdocument *doc, gint start, gint end, gboolean do_scroll)
 void doc_select_line(Tdocument *doc, gint line, gboolean do_scroll);
 gboolean doc_get_selection(Tdocument *doc, gint *start, gint *end);
 gint doc_get_cursor_position(Tdocument *doc);
+void doc_set_statusbar_insovr(Tdocument *doc);
+void doc_set_statusbar_editmode_encoding(Tdocument *doc);
 
 /* the prototype for these functions is changed!! */
 void doc_replace_text_backend(Tdocument *doc, const gchar * newstring, gint start, gint end);
@@ -52,7 +65,11 @@ void doc_insert_two_strings(Tdocument *doc, const gchar *before_str, const gchar
 void doc_bind_signals(Tdocument *doc);
 void doc_unbind_signals(Tdocument *doc);
 
+gchar *ask_new_filename(gchar *oldfilename, gboolean is_move);
+gint doc_save(Tdocument * doc, gint do_save_as, gint do_move);
+
 void doc_new_with_new_file(gchar * new_filename);
+Tdocument *doc_new(gboolean delay_activate);
 gboolean doc_new_with_file(gchar * filename, gboolean delay_activate);
 void docs_new_from_files(GList * file_list);
 void doc_activate(Tdocument *doc);
@@ -77,7 +94,11 @@ void edit_select_all_cb(GtkWidget * widget, gpointer data);
 void doc_toggle_highlighting_cb(gpointer callback_data,guint action,GtkWidget *widget);
 void doc_toggle_wrap_cb(gpointer callback_data,guint action,GtkWidget *widget);
 void doc_toggle_linenumbers_cb(gpointer callback_data,guint action,GtkWidget *widget);
+void all_documents_apply_settings();
 
 void doc_convert_asciichars_in_selection(gpointer callback_data,guint callback_action,GtkWidget *widget);
 void word_count_cb (gpointer callback_data,guint callback_action,GtkWidget *widget);
+void doc_indent_selection(Tdocument *doc, gboolean unindent);
+void menu_indent_cb(gpointer callback_data,guint callback_action, GtkWidget  *widget);
+GList * list_relative_document_filenames(Tdocument *curdoc);
 #endif /* __DOCUMENT_H_ */
