@@ -21,8 +21,9 @@
 #define __FILEBROWSER_H_
 
 void filebrowser_refresh_dir(gchar *dir);
-void filebrowser_open_dir(gchar *dir);
+void filebrowser_open_dir(const gchar *dirarg);
 void filebrowser_filters_rebuild();
 GtkWidget *filebrowser_init();
+void filebrowser_cleanup();
 void filebrowser_scroll_initial();
 #endif /* __FILEBROWSER_H_ */
