@@ -54,7 +54,7 @@ static GList *glist_with_html_tags(gint with_pseudo_classes) {
 	if(main_v->props.allow_h4){
 		tmplist = g_list_append(tmplist, "bdo");
 	}
-	tmplist = g_list_append(tmplist, "bockquote");
+	tmplist = g_list_append(tmplist, "blockquote");
 	tmplist = g_list_append(tmplist, "body");
 	tmplist = g_list_append(tmplist, "br");
 	if(main_v->props.allow_h4){
@@ -350,13 +350,13 @@ static GList *pointer_arr2glist(gchar **arr) {
 	return retlist;
 }
 
-static void cs3d_destroy_lcb(GtkWidget * widget, GdkEvent *event, Tcs3_diag *diag) {
+static void cs3d_destroy_lcb(GtkWidget * widget, Tcs3_diag *diag) {
 	window_destroy(diag->win);
 	g_free(diag);
 }
 
 static void cs3d_cancel_clicked_lcb(GtkWidget * widget, Tcs3_diag *diag) {
-	cs3d_destroy_lcb(NULL, NULL, diag);
+	cs3d_destroy_lcb(NULL, diag);
 }
 
 static void cs3d_ok_clicked_lcb(GtkWidget * widget, Tcs3_diag *diag) {
@@ -441,7 +441,7 @@ static void cs3d_ok_clicked_lcb(GtkWidget * widget, Tcs3_diag *diag) {
 		}
 		g_free(stylebuf);
 	}
-	cs3d_destroy_lcb(NULL, NULL, diag);
+	cs3d_destroy_lcb(NULL, diag);
 }
 
 static void cs3d_select_row_lcb(GtkWidget *clist, gint row, gint column, GdkEventButton *event, Tcs3_diag *diag) {
@@ -560,17 +560,17 @@ static void cs3d_prop_activate_lcb(GtkWidget * widget, Tcs3_diag *diag) {
 			}
 			switch(tmp->buttype) {
 			case but_color:
-				gtk_widget_show(diag->extra_but);
+				gtk_widget_set_sensitive(diag->extra_but, TRUE);
 			break;
 			default:
-				gtk_widget_hide(diag->extra_but);
+				gtk_widget_set_sensitive(diag->extra_but, FALSE);
 			break;
 			}
 		} else {
-				gtk_widget_hide(diag->extra_but);
+				gtk_widget_set_sensitive(diag->extra_but, FALSE);
 		}
 	} else {
-		gtk_widget_hide(diag->extra_but);
+		gtk_widget_set_sensitive(diag->extra_but, FALSE);
 	}
 }
 
@@ -584,25 +584,27 @@ static Tcs3_diag *css_diag(Tcs3_destination dest, Tcs3_style style, GtkWidget *t
 	
 	diag = g_malloc(sizeof(Tcs3_diag));
 	diag->win = window_full(_("Cascading StyleSheet dialog"), GTK_WIN_POS_MOUSE, 
-			5, G_CALLBACK(cs3d_destroy_lcb), diag, TRUE);
+			12, G_CALLBACK(cs3d_destroy_lcb), diag, TRUE);
 	gtk_window_set_role(GTK_WINDOW(diag->win), "css");
 	diag->dest = dest;
 	diag->styletype = style;
 	diag->grab = grab;
 	diag->selected_row = -1;
 
-	vbox = gtk_vbox_new(FALSE, 5);
+	vbox = gtk_vbox_new(FALSE, 0);
 	gtk_container_add(GTK_CONTAINER(diag->win),vbox);
 	
-	table = gtk_table_new(3,3,0);
+	table = gtk_table_new(3, 6, FALSE);
+	gtk_table_set_row_spacings(GTK_TABLE(table), 12);
+	gtk_table_set_col_spacings(GTK_TABLE(table), 12);
 	gtk_box_pack_start(GTK_BOX(vbox), table, FALSE, FALSE, 0);
 	tmplist = NULL;
 	
 	if (diag->styletype == multistyle) {
 		tmplist = glist_with_html_tags(1);
 		diag->selector = combo_with_popdown(NULL, tmplist,1);
-		gtk_table_attach_defaults(GTK_TABLE(table), gtk_label_new(_("selector")), 0,1,0,1);
-		gtk_table_attach_defaults(GTK_TABLE(table), diag->selector, 1,2,0,1);
+		bf_mnemonic_label_tad_with_alignment(_("_Selector:"), diag->selector, 0, 0.5, table, 0, 1, 0, 1);
+		gtk_table_attach_defaults(GTK_TABLE(table), diag->selector, 1 ,5 , 0, 1);
 		g_list_free(tmplist);
 		tmplist = NULL;
 	} 
@@ -620,19 +622,19 @@ static Tcs3_diag *css_diag(Tcs3_destination dest, Tcs3_style style, GtkWidget *t
 	gtk_signal_connect(GTK_OBJECT(GTK_COMBO(diag->property)->entry), "changed", G_CALLBACK(cs3d_prop_activate_lcb), diag);
 
 	diag->value = combo_with_popdown(NULL, tmplist,1);
-	gtk_table_attach_defaults(GTK_TABLE(table), gtk_label_new(_("property")), 0,1,1,2);
-	gtk_table_attach_defaults(GTK_TABLE(table), diag->property, 1,2,1,2);
-	gtk_table_attach_defaults(GTK_TABLE(table), gtk_label_new(_("value")), 0,1,2,3);
-	gtk_table_attach_defaults(GTK_TABLE(table), diag->value, 1,2,2,3);
-
+	bf_mnemonic_label_tad_with_alignment(_("_Property:"), diag->property, 0, 0.5, table, 0, 1, 1, 2);
+	gtk_table_attach_defaults(GTK_TABLE(table), diag->property, 1, 5, 1, 2);
+	bf_mnemonic_label_tad_with_alignment(_("_Value:"), diag->value, 0, 0.5, table, 0, 1, 2, 3);
+	gtk_table_attach_defaults(GTK_TABLE(table), diag->value, 1, 5, 2, 3);
+	
 	gtk_widget_realize(diag->win);
 
 	diag->extra_but = color_but_new(GTK_WIDGET(GTK_COMBO(diag->value)->entry), diag->win);
-	gtk_table_attach_defaults(GTK_TABLE(table), diag->extra_but, 2,3,2,3);
+	gtk_table_attach(GTK_TABLE(table), diag->extra_but, 5, 6, 2, 3, GTK_EXPAND, GTK_EXPAND, 0, 0);
 
 	/* the list widget and the buttons are in a horizontal box */
-	hbox = gtk_hbox_new(FALSE, 0);
-	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
+	hbox = gtk_hbox_new(FALSE, 12);
+	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 18);
 
 	scrolwin = gtk_scrolled_window_new(NULL, NULL);
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolwin), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
@@ -655,25 +657,32 @@ static Tcs3_diag *css_diag(Tcs3_destination dest, Tcs3_style style, GtkWidget *t
 
 	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrolwin), diag->clist);
 
-	vbox2 = gtk_vbox_new(FALSE, 0);
+	vbox2 = gtk_vbox_new(FALSE, 6);
 	gtk_box_pack_start(GTK_BOX(hbox), vbox2, FALSE, FALSE, 0);
 	
-	but = bf_stock_button(_(" Add "), G_CALLBACK(cs3d_add_clicked_lcb), diag);
+	but = bf_stock_button(_(" _Add "), G_CALLBACK(cs3d_add_clicked_lcb), diag);
 	gtk_box_pack_start(GTK_BOX(vbox2), but, FALSE, FALSE, 0);
 
-	but = bf_stock_button(_(" Update "), G_CALLBACK(cs3d_update_clicked_lcb), diag);
+	but = bf_stock_button(_(" _Update "), G_CALLBACK(cs3d_update_clicked_lcb), diag);
 	gtk_box_pack_start(GTK_BOX(vbox2), but, FALSE, FALSE, 0);
 	
-	but = bf_stock_button(_(" Delete "), G_CALLBACK(cs3d_del_clicked_lcb), diag);
+	but = bf_stock_button(_(" _Delete "), G_CALLBACK(cs3d_del_clicked_lcb), diag);
 	gtk_box_pack_start(GTK_BOX(vbox2), but, FALSE, FALSE, 0);
 
 	/* the ok and cancel button are in a horizontal box below */
-	hbox = gtk_hbox_new(TRUE, 0);
-	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
+	hbox = gtk_hbox_new(FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(hbox), gtk_hseparator_new(), TRUE, TRUE, 0);
+	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 12);	
+
+	hbox = gtk_hbutton_box_new();
+	gtk_hbutton_box_set_layout_default(GTK_BUTTONBOX_END);
+	gtk_button_box_set_spacing(GTK_BUTTON_BOX(hbox), 12);
+	
 	but = bf_stock_cancel_button(G_CALLBACK(cs3d_cancel_clicked_lcb), diag);
-	gtk_box_pack_start(GTK_BOX(hbox), but, TRUE, TRUE, 0);
+	gtk_box_pack_start(GTK_BOX(hbox), but, FALSE, FALSE, 0);
 	but = bf_stock_ok_button(G_CALLBACK(cs3d_ok_clicked_lcb), diag);
-	gtk_box_pack_start(GTK_BOX(hbox), but, TRUE, TRUE, 0);
+	gtk_box_pack_start(GTK_BOX(hbox), but, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);	
 	
 	gtk_widget_show_all(diag->win);
 	
@@ -913,9 +922,8 @@ GtkWidget *style_but_new(GtkWidget * which_entry, GtkWidget * win)
 	style_but = gtk_button_new();
 	hbox = gtk_hbox_new(FALSE, 0);
 	gtk_box_pack_start(GTK_BOX(hbox), new_pixmap(92),FALSE, FALSE, 3);
-	gtk_box_pack_start(GTK_BOX(hbox), gtk_label_new(_("Style")),TRUE, TRUE, 3);
+	gtk_box_pack_start(GTK_BOX(hbox), gtk_label_new_with_mnemonic(_("_Style...")), TRUE, TRUE, 3);
 	gtk_container_add(GTK_CONTAINER(style_but), hbox);
-
 	gtk_signal_connect(GTK_OBJECT(style_but), "clicked", G_CALLBACK(style_but_clicked_lcb), which_entry);
 	gtk_widget_show_all(style_but);
 	return style_but;
@@ -950,10 +958,9 @@ GtkWidget *style_but_new_for_wizard(GtkWidget * textview) {
 
 	style_but = gtk_button_new();
 	hbox = gtk_hbox_new(FALSE, 0);
-	gtk_box_pack_start(GTK_BOX(hbox), new_pixmap(92),FALSE, FALSE, 3);
-	gtk_box_pack_start(GTK_BOX(hbox), gtk_label_new(_("Style")),TRUE, TRUE, 3);
+	gtk_box_pack_start(GTK_BOX(hbox), new_pixmap(92),FALSE, FALSE, 6);
+	gtk_box_pack_start(GTK_BOX(hbox), gtk_label_new_with_mnemonic(_("_Style...")), TRUE, TRUE, 6);
 	gtk_container_add(GTK_CONTAINER(style_but), hbox);
-
 	gtk_signal_connect(GTK_OBJECT(style_but), "clicked", G_CALLBACK(style_but_for_wizard_clicked_lcb), textview);
 	gtk_widget_show_all(style_but);
 	return style_but;
@@ -1060,7 +1067,7 @@ static gdouble *hex_to_gdouble_arr(gchar *color)
 	return tmpcol;
 }
  
-static void colsel_destroy_lcb(GtkWidget *widget, GdkEvent *event, Tcolsel *csd) {
+static void colsel_destroy_lcb(GtkWidget *widget, Tcolsel *csd) {
 	if (csd->is_modal) {
 		gtk_main_quit();
 	} else {
@@ -1082,12 +1089,12 @@ static void colsel_destroy_lcb(GtkWidget *widget, GdkEvent *event, Tcolsel *csd)
 }
 
 static void colsel_ok_clicked_lcb(GtkWidget *widget, Tcolsel *csd) { 
-	colsel_destroy_lcb(NULL, NULL, csd);
+	colsel_destroy_lcb(NULL, csd);
 }
 
 static void colsel_cancel_clicked_lcb(GtkWidget *widget, Tcolsel *csd) {
 	gtk_entry_set_text(GTK_ENTRY(csd->hexentry), "");
-	colsel_destroy_lcb(NULL, NULL, csd);
+	colsel_destroy_lcb(NULL, csd);
 }
 /* declaration needed to connect/disconnect callback */
 static void colsel_color_changed(GtkWidget *widget, Tcolsel *csd);
@@ -1135,7 +1142,7 @@ static Tcolsel *colsel_dialog(gchar *setcolor, gint modal, gint startpos, gint e
 	csd->startpos = startpos;
 	csd->endpos = endpos;
 	DEBUG_MSG("colsel_dialog, malloced at %p\n", csd);
-	csd->win = window_full(_("Bluefish: Select color"), GTK_WIN_POS_MOUSE, 5, G_CALLBACK(colsel_destroy_lcb), csd, TRUE);
+	csd->win = window_full2(_("Bluefish: Select color"), GTK_WIN_POS_MOUSE, 12, G_CALLBACK(colsel_destroy_lcb), csd, TRUE, NULL);
 	vbox = gtk_vbox_new(FALSE, 0);
 	gtk_container_add(GTK_CONTAINER(csd->win), vbox);
 	csd->csel = gtk_color_selection_new();
@@ -1151,15 +1158,19 @@ static Tcolsel *colsel_dialog(gchar *setcolor, gint modal, gint startpos, gint e
 	gtk_box_pack_start(GTK_BOX(vbox), csd->csel, TRUE, TRUE, 0);
 
 	hbox = gtk_hbox_new(FALSE, 0);
-	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 12);
 	
 	csd->hexentry = boxed_entry_with_text(this_color, 7, hbox);
 	csd->hex_changed_id = gtk_signal_connect(GTK_OBJECT(csd->hexentry), "changed", G_CALLBACK(hexentry_color_changed), csd);
-	csd->websafe = boxed_checkbut_with_value(_("websafe"), 0, hbox);
+	csd->websafe = boxed_checkbut_with_value(_("_websafe"), 0, hbox);
+
+	hbox = gtk_hbox_new(FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(hbox), gtk_hseparator_new(), TRUE, TRUE, 0);
+	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 12);
 
 	hbox = gtk_hbutton_box_new();
 	gtk_hbutton_box_set_layout_default(GTK_BUTTONBOX_END);
-	gtk_button_box_set_spacing(GTK_BUTTON_BOX(hbox), 1);
+	gtk_button_box_set_spacing(GTK_BUTTON_BOX(hbox), 12);
 	gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
 
 	but = bf_stock_cancel_button(G_CALLBACK(colsel_cancel_clicked_lcb), csd);
