@@ -41,8 +41,11 @@ typedef struct {
 	guint adj_changed_id;
 } Timage_diag;
 
-void image_diag_destroy_cb(GtkWidget * widget, GdkEvent *event,  Timage_diag *imdg) {
-	html_diag_destroy_cb(widget,event,imdg->dg);
+void image_diag_destroy_cb(GtkWidget * widget, Timage_diag *imdg) {
+	html_diag_destroy_cb(widget,imdg->dg);
+	if (imdg->pb) {
+		g_object_unref(imdg->pb);
+	}
 	g_free(imdg);
 }
 
@@ -66,7 +69,7 @@ static void image_insert_dialogok_lcb(GtkWidget * widget, Timage_diag *imdg) {
 			} else {
 				gdk_pixbuf_save(tmp_im,thumbnailfilename,main_v->props.image_thumbnailtype,&error, NULL);
 			}
-			gdk_pixbuf_unref (tmp_im);
+			g_object_unref (tmp_im);
 			if (error) {
 				g_print("ERROR while saving thumbnail: %s\n", error->message);
 				g_error_free(error);
@@ -86,7 +89,7 @@ static void image_insert_dialogok_lcb(GtkWidget * widget, Timage_diag *imdg) {
 	thestring = insert_integer_if_spin(imdg->dg->spin[3], cap("HSPACE"), thestring, NULL);
 	thestring = insert_integer_if_spin(imdg->dg->spin[4], cap("VSPACE"), thestring, NULL);
 	thestring = insert_string_if_entry(imdg->dg->entry[1], cap("NAME"), thestring, NULL);
-	thestring = insert_string_if_entry(imdg->dg->entry[2], cap("ALT"), thestring, NULL);
+	thestring = insert_string_if_entry(imdg->dg->entry[2], cap("ALT"), thestring, "");
 	thestring = insert_string_if_entry(imdg->dg->entry[3], cap("USEMAP"), thestring, NULL);
 	thestring = insert_string_if_entry(GTK_WIDGET(GTK_COMBO(imdg->dg->combo[0])->entry), cap("ALIGN"), thestring, NULL);
 	thestring = insert_string_if_entry(imdg->dg->entry[4], NULL, thestring, NULL);
@@ -101,19 +104,22 @@ static void image_insert_dialogok_lcb(GtkWidget * widget, Timage_diag *imdg) {
 	}
 
 	g_free(finalstring);
-	image_diag_destroy_cb(NULL, NULL, imdg);	
+	image_diag_destroy_cb(NULL, imdg);	
 }
 
 void image_diag_cancel_clicked_cb(GtkWidget *widget, gpointer data) {
-	image_diag_destroy_cb(NULL, NULL, data);
+	image_diag_destroy_cb(NULL, data);
 }
 
 static void image_diag_finish(Timage_diag *imdg, GtkSignalFunc ok_func) {
 	GtkWidget *hbox;
 
+	hbox = gtk_hbox_new(FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(hbox), gtk_hseparator_new(), TRUE, TRUE, 0);
+	gtk_box_pack_start(GTK_BOX(imdg->dg->vbox), hbox, FALSE, FALSE, 12);
 	hbox = gtk_hbutton_box_new();
 	gtk_hbutton_box_set_layout_default(GTK_BUTTONBOX_END);
-	gtk_button_box_set_spacing(GTK_BUTTON_BOX(hbox), 1);
+	gtk_button_box_set_spacing(GTK_BUTTON_BOX(hbox), 12);
 	
 	imdg->dg->obut = bf_stock_ok_button(ok_func, imdg);
 	imdg->dg->cbut = bf_stock_cancel_button(G_CALLBACK(image_diag_cancel_clicked_cb), imdg);
@@ -129,13 +135,13 @@ static void image_diag_finish(Timage_diag *imdg, GtkSignalFunc ok_func) {
 static void image_filename_changed(GtkWidget * widget, Timage_diag *imdg) {
 	gint pb_width, pd_height, toobig;
 	GdkPixbuf *tmp_pb;
+	DEBUG_MSG("image_filename_changed() started. GTK_IS_WIDGET(imdg->im) == %d\n", GTK_IS_WIDGET(imdg->im));
 	if (imdg->pb) {
-		gdk_pixbuf_unref(imdg->pb);
-	}
-	if (imdg->im) {
-		gtk_widget_destroy(imdg->im);
+		g_object_unref(imdg->pb);
 	}
+	
 	imdg->pb = gdk_pixbuf_new_from_file(gtk_entry_get_text(GTK_ENTRY(imdg->dg->entry[0])), NULL);
+	
 	if (!imdg->pb) {
 		return;
 	}
@@ -160,23 +166,31 @@ static void image_filename_changed(GtkWidget * widget, Timage_diag *imdg) {
 	}
 	
 	tmp_pb = gdk_pixbuf_scale_simple(imdg->pb, (pb_width / toobig), (pd_height / toobig), main_v->props.image_thumbnail_refresh_quality ? GDK_INTERP_BILINEAR : GDK_INTERP_NEAREST);
+	
+	if (GTK_IS_WIDGET(imdg->im)) {
+		DEBUG_MSG("imdg->im == %p\n", imdg->im);
+		DEBUG_MSG("gtk_widget_destroy() %p\n", imdg->im);
+		gtk_widget_destroy(imdg->im);
+	}
+	
 	imdg->im = gtk_image_new_from_pixbuf(tmp_pb);
-	gdk_pixbuf_unref(tmp_pb);
+
+	g_object_unref(tmp_pb);
 	gtk_container_add(GTK_CONTAINER(imdg->frame), imdg->im);
 	gtk_widget_show(imdg->im);
+	DEBUG_MSG("imdg->im == %p\n", imdg->im);
+	DEBUG_MSG("image_filename_changed() finished. GTK_IS_WIDGET(imdg->im) == %d\n", GTK_IS_WIDGET(imdg->im));
 }
 
-
 static void image_adjust_changed(GtkAdjustment * adj, Timage_diag *imdg) {
 	GdkPixbuf *tmp_pb;
 	gint tn_width, tn_height;
+	DEBUG_MSG("image_adjust_changed started. GTK_IS_WIDGET(imdg->im) == %d\n", GTK_IS_WIDGET(imdg->im));
 	if (!imdg->pb) {
 		image_filename_changed(NULL, imdg);
 		return;
 	}
-	if (imdg->im) {
-		gtk_widget_destroy(imdg->im);
-	}
+
 	tn_width = imdg->adjustment->value * gdk_pixbuf_get_width(imdg->pb);
 	tn_height = imdg->adjustment->value * gdk_pixbuf_get_height(imdg->pb);
 
@@ -184,17 +198,25 @@ static void image_adjust_changed(GtkAdjustment * adj, Timage_diag *imdg) {
 	gtk_spin_button_set_value(GTK_SPIN_BUTTON(imdg->dg->spin[1]),tn_height);
 
 	tmp_pb = gdk_pixbuf_scale_simple(imdg->pb, tn_width, tn_height, main_v->props.image_thumbnail_refresh_quality ? GDK_INTERP_BILINEAR : GDK_INTERP_NEAREST);
+	
+	if (GTK_IS_WIDGET(imdg->im)) {
+		DEBUG_MSG("imdg->im == %p\n", imdg->im);
+		DEBUG_MSG("gtk_widget_destroy() %p\n", imdg->im);		
+		gtk_widget_destroy(imdg->im);
+	}
+	
 	imdg->im = gtk_image_new_from_pixbuf(tmp_pb);
-	gdk_pixbuf_unref(tmp_pb);
+	g_object_unref(tmp_pb);
 	gtk_container_add(GTK_CONTAINER(imdg->frame), imdg->im);
 	gtk_widget_show(imdg->im);
+	DEBUG_MSG("image_adjust_changed finished. GTK_IS_WIDGET(imdg->im) == %d\n", GTK_IS_WIDGET(imdg->im));
 }
 
 void image_insert_dialog_backend(gchar *filename,GtkWidget *widget, gpointer data, gboolean is_thumbnail) {
 	gchar *tagvalues[11];
 	gchar *custom = NULL;
 	Timage_diag *imdg;
-	
+	GList *popuplist = NULL;
 	GtkWidget *dgtable;
 
 	imdg = g_new(Timage_diag, 1);
@@ -202,9 +224,9 @@ void image_insert_dialog_backend(gchar *filename,GtkWidget *widget, gpointer dat
 	imdg->im = NULL;
 	imdg->is_thumbnail = is_thumbnail;
 	if (is_thumbnail) {
-		imdg->dg = html_diag_new(_("Insert image"));
-	} else {
 		imdg->dg = html_diag_new(_("Insert thumbnail"));
+	} else {
+		imdg->dg = html_diag_new(_("Insert image"));
 	}
 	{
 		static gchar *tagitems[] = { "width", "height", "alt", "border", "src", "hspace", "vspace", "align", "name", "usemap", NULL };
@@ -231,59 +253,65 @@ void image_insert_dialog_backend(gchar *filename,GtkWidget *widget, gpointer dat
 	} else {
 		imdg->dg->entry[0] = entry_with_text(tagvalues[4], 1024);
 	}
-	gtk_table_attach_defaults(GTK_TABLE(dgtable), gtk_label_new(_("Image location")), 0, 1, 0, 1);
+	bf_mnemonic_label_tad_with_alignment(_("_Image location:"), imdg->dg->entry[0], 0, 0.5, dgtable, 0, 1, 0, 1); 
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), imdg->dg->entry[0], 1, 7, 0, 1);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), file_but_new(imdg->dg->entry[0], imdg->dg->dialog, 0), 7, 9, 0, 1);
-	g_signal_connect(G_OBJECT(imdg->dg->entry[0]), "activate", G_CALLBACK(image_filename_changed), imdg);
+	g_signal_connect(G_OBJECT(imdg->dg->entry[0]), "changed", G_CALLBACK(image_filename_changed), imdg);
 
 	imdg->dg->spin[0] = spinbut_with_value(NULL, 0, 5000, 1.0, 10.0);
 	imdg->dg->check[0] = gtk_check_button_new_with_label("%");
 	parse_integer_for_dialog(tagvalues[0], imdg->dg->spin[0] , NULL, imdg->dg->check[0]);
-	gtk_table_attach_defaults(GTK_TABLE(dgtable), gtk_label_new(_("Width")), 6, 7, 1, 2);
+	bf_mnemonic_label_tad_with_alignment(_("_Width:"), imdg->dg->spin[0], 0, 0.5, dgtable, 6, 7, 1, 2);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), imdg->dg->spin[0], 7, 8, 1, 2);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), imdg->dg->check[0], 8, 9, 1, 2);
 
 	imdg->dg->spin[1] = spinbut_with_value(NULL, 0, 5000, 1.0, 10.0);
 	imdg->dg->check[1] = gtk_check_button_new_with_label("%");
 	parse_integer_for_dialog(tagvalues[1], imdg->dg->spin[1] , NULL, imdg->dg->check[1]);
-	gtk_table_attach_defaults(GTK_TABLE(dgtable), gtk_label_new(_("Height")), 6, 7, 2, 3);
+	bf_mnemonic_label_tad_with_alignment(_("Hei_ght:"), imdg->dg->spin[1], 0, 0.5, dgtable, 6, 7, 2, 3);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), imdg->dg->spin[1], 7, 8, 2, 3);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), imdg->dg->check[1], 8, 9, 2, 3);
 
 	imdg->dg->entry[3] = entry_with_text(tagvalues[9], 1024);
-	gtk_table_attach_defaults(GTK_TABLE(dgtable), gtk_label_new(_("Usemap")), 0, 1, 1, 2);
+	bf_mnemonic_label_tad_with_alignment(_("_Usemap:"), imdg->dg->entry[3], 0, 0.5, dgtable, 0, 1, 1, 2);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), imdg->dg->entry[3], 1, 3, 1, 2);
 
 	imdg->dg->entry[1] = entry_with_text(tagvalues[8], 1024);
-	gtk_table_attach_defaults(GTK_TABLE(dgtable), gtk_label_new(_("Name")), 0, 1, 2, 3);
+	bf_mnemonic_label_tad_with_alignment(_("_Name:"), imdg->dg->entry[1], 0, 0.5, dgtable, 0, 1, 2, 3);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), imdg->dg->entry[1], 1, 3, 2, 3);
 
 	imdg->dg->entry[2] = entry_with_text(tagvalues[2], 1024);
-	gtk_table_attach_defaults(GTK_TABLE(dgtable), gtk_label_new(_("Alternate text")), 0, 1, 3, 4);
+	bf_mnemonic_label_tad_with_alignment(_("Alternate _text:"), imdg->dg->entry[2], 0, 0.5, dgtable, 0, 1, 3, 4);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), imdg->dg->entry[2], 1, 6, 3, 4);
 
 	imdg->dg->entry[4] = entry_with_text(custom, 1024);
-	gtk_table_attach_defaults(GTK_TABLE(dgtable), gtk_label_new(_("Custom")), 0, 1, 4, 5);
+	bf_mnemonic_label_tad_with_alignment(_("Custo_m:"), imdg->dg->entry[4], 0, 0.5, dgtable, 0, 1, 4, 5);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), imdg->dg->entry[4], 1, 6, 4, 5);
 
 	imdg->dg->spin[3] = spinbut_with_value(tagvalues[5], 0, 500, 1.0, 5.0);
-	gtk_table_attach_defaults(GTK_TABLE(dgtable), gtk_label_new(_("Hspace")), 6, 7, 3, 4);
+	bf_mnemonic_label_tad_with_alignment(_("_Hspace:"), imdg->dg->spin[3], 0, 0.5, dgtable, 6, 7, 3, 4);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), imdg->dg->spin[3], 7, 9, 3, 4);
 
 	imdg->dg->spin[4] = spinbut_with_value(tagvalues[6], 0, 500, 1.0, 5.0);
-	gtk_table_attach_defaults(GTK_TABLE(dgtable), gtk_label_new(_("Vspace")), 6, 7, 4, 5);
+	bf_mnemonic_label_tad_with_alignment(_("_Vspace:"), imdg->dg->spin[4], 0, 0.5, dgtable, 6, 7, 4, 5);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), imdg->dg->spin[4], 7, 9, 4, 5);
 
-	imdg->dg->combo[0] = combo_with_popdown_sized(tagvalues[7], recent_attribs.positionlist, 1, 90);
-	gtk_table_attach_defaults(GTK_TABLE(dgtable), gtk_label_new(_("Align")), 3, 4, 1, 2);
+	popuplist = g_list_append(NULL, "bottom");
+	popuplist = g_list_append(popuplist, "middle");
+	popuplist = g_list_append(popuplist, "top");
+	popuplist = g_list_append(popuplist, "left");
+	popuplist = g_list_append(popuplist, "right");
+	imdg->dg->combo[0] = combo_with_popdown_sized(tagvalues[7], popuplist, 1, 90);
+	g_list_free(popuplist);
+	bf_mnemonic_label_tad_with_alignment(_("_Align:"), imdg->dg->combo[0], 0, 0.5, dgtable, 3, 4, 1, 2);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), imdg->dg->combo[0], 4, 6, 1, 2);
 
 	imdg->dg->spin[2] = spinbut_with_value(tagvalues[3], 0, 500, 1.0, 5.0);
-	gtk_table_attach_defaults(GTK_TABLE(dgtable),gtk_label_new(_("Border")) , 3, 4, 2, 3);
+	bf_mnemonic_label_tad_with_alignment(_("Borde_r:"), imdg->dg->spin[2], 0, 0.5, dgtable, 3, 4, 2, 3);
 	gtk_table_attach_defaults(GTK_TABLE(dgtable), imdg->dg->spin[2], 4, 6, 2, 3);
 
 	if (filename || tagvalues[4]) {
-		g_signal_emit_by_name(G_OBJECT(imdg->dg->entry[0]), "activate");
+		g_signal_emit_by_name(G_OBJECT(imdg->dg->entry[0]), "changed");
 	}
 
 	image_diag_finish(imdg, G_CALLBACK(image_insert_dialogok_lcb));
@@ -297,6 +325,9 @@ void image_insert_dialog_cb(GtkWidget * widget, gpointer data) {
 void thumbnail_insert_dialog_cb(GtkWidget * widget, gpointer data) {
 	image_insert_dialog_backend(NULL, NULL, NULL, TRUE);
 }
+void image_insert_from_filename(gchar *filename) {
+	image_insert_dialog_backend(filename,NULL, NULL, FALSE);
+}
 
 typedef struct {
 	GtkWidget *win;
@@ -343,9 +374,15 @@ static void multi_thumbnail_ok_clicked(GtkWidget *widget, Tmuthudia *mtd) {
 		GdkPixbuf *tmp_im1, *tmp_im2;
 		gint tw,th,ow,oh;
 		GError *error=NULL;
-		gchar *thumbfilename, *filename=(gchar *)tmplist->data;
+		gchar *thumbfilename, *filename=(gchar *)tmplist->data, *relfilename;
 	
-		thumbfilename = create_thumbnail_filename(filename);
+		if (main_v->current_document->filename) {
+			relfilename = create_relative_link_to(main_v->current_document->filename, filename);
+		} else {
+			relfilename = g_path_get_basename(filename);
+		}
+		thumbfilename = create_thumbnail_filename(relfilename);
+		g_free(relfilename);
 
 		DEBUG_MSG("filename=%s, thumbfilename=%s\n", filename, thumbfilename);
 		tmp_im1 = gdk_pixbuf_new_from_file(filename, NULL);
@@ -467,7 +504,7 @@ static void multi_thumbnail_radio_toggled_lcb(GtkToggleButton *togglebutton,Tmut
 void multi_thumbnail_dialog_cb(GtkWidget * widget, gpointer data) {
 	Tmuthudia *mtd;
 	GtkWidget *vbox, *hbox, *but, *table, *label, *scrolwin, *textview;
-	
+	gint tb;
 	
 	mtd = g_new(Tmuthudia, 1);
 	mtd->win = window_full(_("Multi thumbnail"), GTK_WIN_POS_MOUSE, 5, G_CALLBACK(multi_thumbnail_dialog_destroy), mtd, TRUE);
@@ -490,10 +527,7 @@ void multi_thumbnail_dialog_cb(GtkWidget * widget, gpointer data) {
 	g_signal_connect(G_OBJECT(mtd->radio[3]), "toggled", G_CALLBACK(multi_thumbnail_radio_toggled_lcb), mtd);
 	gtk_spin_button_set_value(GTK_SPIN_BUTTON(mtd->spins[0]), main_v->props.image_thumbnailsizing_val1);
 	gtk_spin_button_set_value(GTK_SPIN_BUTTON(mtd->spins[1]), main_v->props.image_thumbnailsizing_val2);
-	if (main_v->props.image_thumbnailsizing_type > 4) {
-		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(mtd->radio[main_v->props.image_thumbnailsizing_type]), TRUE);
-	}
-	
+
 	gtk_table_attach_defaults(GTK_TABLE(table), mtd->radio[0], 0, 1, 0, 1);
 	gtk_table_attach_defaults(GTK_TABLE(table), mtd->radio[1], 0, 1, 1, 2);
 	gtk_table_attach_defaults(GTK_TABLE(table), mtd->radio[2], 0, 1, 2, 3);
@@ -522,4 +556,8 @@ void multi_thumbnail_dialog_cb(GtkWidget * widget, gpointer data) {
 	gtk_box_pack_start(GTK_BOX(hbox), but,FALSE, FALSE, 5);
 	gtk_window_set_default(GTK_WINDOW(mtd->win), but);
 	gtk_widget_show_all(mtd->win);
+
+	tb = main_v->props.image_thumbnailsizing_type < 4 ? main_v->props.image_thumbnailsizing_type : 0;
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(mtd->radio[tb]), TRUE);
+	multi_thumbnail_radio_toggled_lcb(GTK_TOGGLE_BUTTON(mtd->radio[tb]), mtd);
 }
