@@ -94,7 +94,7 @@ static Tpixmap tp[] = {
 	{81, meta},
 	{82, embed},
 	{83, color_select},
-/*	{84, time}, a function called time already exist, we cannot have a variable called time */
+	{84, php3}, 
 	{85, cssnewstyle},
 	{86, cssspan},
 	{87, cssdiv},
@@ -115,6 +115,11 @@ static Tpixmap tp[] = {
 	{102, view_in_browser},
 	{103, context}, /* context dropdown image */
 	{104, spellcheck},
+	{105, filetree},
+	{106, fref},
+	{110, indent},
+	{111, unindent},
+	{112, open_small},
 	{150,prefseditor},
 	{151,prefsexternal},
 	{152,prefsfiles},
@@ -124,7 +129,7 @@ static Tpixmap tp[] = {
 	{156,prefsinterface},
 	{157,prefsoutputparser},
 	{158,prefssyntax},
-	{104, nothing}
+	{159, nothing}
 };
 
 GtkWidget *new_pixmap(gint type) {
