@@ -1,4 +1,4 @@
-/* #define DEBUG */
+/*#define DEBUG*/
 
 #include <gtk/gtk.h>
 #include <string.h> /* strcmp() */
@@ -7,7 +7,12 @@
 #include "stringlist.h" /* duplicate_arraylist*/
 #include "bf_lib.h" /* list_switch_order() */
 #include "gtk_easy.h"
+#include "document.h"
 #include "pixmap.h"
+#include "highlight.h"
+#include "filebrowser.h"
+#include "menu.h"
+#include "gui.h"
 
 enum {
 	view_line_numbers,
@@ -18,6 +23,7 @@ enum {
 	filebrowser_dir_icon,
 	editor_font_string,		/* editor font */
 	editor_tab_width,	/* editor tabwidth */
+	editor_indent_wspaces,
 	tab_font_string,		/* notebook tabs font */
 	highlight_num_lines_count, /* number of lines to highlight in continous highlighting */	
 	defaulthighlight,		/* highlight documents by default */
@@ -30,7 +36,8 @@ enum {
 	max_dir_history,	/* length of directory history */
 	backup_file, 			/* wheather to use a backup file */
 	backup_filestring,  /* the string to append to the backup file */
-	backup_abort_style, /* if the backup fails, continue 'save', 'abort' save, or 'ask' user */
+	backup_abort_action, /* if the backup fails, continue 'save', 'abort' save, or 'ask' user */
+	backup_cleanuponclose, /* remove the backupfile after close ? */
 	image_thumbnailstring,	/* string to append to thumbnail filenames */
 	image_thumbnailtype,	/* fileformat to use for thumbnails, "jpeg" or "png" can be handled by gdkpixbuf*/
 	image_thumbnailsizing_type,	/* scaling ratio=0, fixed width=1, height=2, width+height (discard aspect ratio)=3 */
@@ -46,12 +53,12 @@ enum {
 	ext_browsers_in_submenu,
 	ext_commands_in_submenu,
 	ext_outputbox_in_submenu,
+	document_tabposition,
+	leftpanel_tabposition,
 	default_advanced_snr,
 	cont_highlight_full, 	/* if you want to highlight the full text or just the line */
 	/* not yet in use */
 	image_editor_cline, 	/* image editor commandline */
-	cfg_weblint_cline,	/* weblint command line */
-	cfg_tab_pos,			/* notebook tabs positioning */
 	full_p,				/* use </p> */
 	full_li,				/* use </li> */
 	allow_css,				/* CSS allowed */
@@ -113,7 +120,7 @@ enum {
 typedef struct {
 	GtkListStore *lstore;
 	GtkWidget *lview;
-	GtkWidget *entry[5];
+	GtkWidget *entry[6];
 	gchar **curstrarr;
 } Tfiletypedialog;
 
@@ -230,7 +237,6 @@ static GtkWidget *prefs_combo(const gchar *title, const gchar *curval, GtkWidget
 	gtk_box_pack_start(GTK_BOX(hbox), gtk_label_new(title), FALSE, FALSE, 3);
 	return_widget = boxed_combo_with_popdown(curval, poplist, editable, hbox);
 	return return_widget;
-
 }
 
 static GtkWidget *prefs_integer(const gchar *title, const gint curval, GtkWidget *box, Tprefdialog *pd, gfloat lower, gfloat upper) {
@@ -349,14 +355,16 @@ static gchar **filetype_create_strarr(Tprefdialog *pd) {
 	gchar *escaped, *tmp;
 	tmp = gtk_editable_get_chars(GTK_EDITABLE(pd->ftd.entry[0]),0,-1);
 	if (strlen(tmp)) {
-		strarr = g_malloc(5*sizeof(gchar *));
+		strarr = g_malloc(7*sizeof(gchar *));
 		strarr[0] = tmp;
 		strarr[1] = gtk_editable_get_chars(GTK_EDITABLE(pd->ftd.entry[1]),0,-1);
 		escaped = gtk_editable_get_chars(GTK_EDITABLE(pd->ftd.entry[2]),0,-1);
 		strarr[2] = unescapestring(escaped);
 		g_free(escaped);
 		strarr[3] = gtk_editable_get_chars(GTK_EDITABLE(pd->ftd.entry[3]),0,-1);
-		strarr[4] = NULL;
+		strarr[4] = g_strdup(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(pd->ftd.entry[4])) ? "1" : "0");
+		strarr[5] = gtk_editable_get_chars(GTK_EDITABLE(pd->ftd.entry[5]),0,-1);
+		strarr[6] = NULL;
 		DEBUG_MSG("filetype_create_strarr, created at %p\n", strarr);
 		return strarr;
 	} else {
@@ -365,24 +373,37 @@ static gchar **filetype_create_strarr(Tprefdialog *pd) {
 	}
 }
 
+static void set_filetype_strarr_in_list(GtkTreeIter *iter, gchar **strarr, Tprefdialog *pd) {
+	gint arrcount;
+	arrcount = count_array(strarr);
+	if (arrcount==6) {
+		gchar *escaped = escapestring(strarr[2],'\0');
+		DEBUG_MSG("set_filetype_strarr_in_list, 4=%d, string was %s\n",(strarr[4][0] != '0'), strarr[4]);
+		gtk_list_store_set(GTK_LIST_STORE(pd->ftd.lstore), iter
+			,0,strarr[0]
+			,1,strarr[1]
+			,2,escaped
+			,3,strarr[3]
+			,4,(strarr[4][0] != '0')
+			,5,strarr[5]
+			,-1);
+		g_free(escaped);
+	}
+}
+
 static void add_new_filetype_lcb(GtkWidget *wid, Tprefdialog *pd) {
 /*	add_new_general_lcb(pd, pd->ftd.entry[0],4,filetypes,GTK_LIST_STORE(pd->ftd.lstore));*/
 	gchar **strarr;
 	strarr = filetype_create_strarr(pd);
 	if (strarr) {
+		GtkTreeIter iter;
+		pd->ftd.curstrarr = strarr;
 		pd->lists[filetypes] = g_list_append(pd->lists[filetypes], strarr);
-		{
-			GtkTreeIter iter;
-			gchar *escaped = escapestring(strarr[2],'\0');
-			gtk_list_store_append(GTK_LIST_STORE(pd->ftd.lstore), &iter);
-			gtk_list_store_set(GTK_LIST_STORE(pd->ftd.lstore), &iter
-				,0,strarr[0]
-				,1,strarr[1]
-				,2,escaped
-				,3,strarr[3]
-				,-1);
-			g_free(escaped);
-		}
+		DEBUG_MSG("add_new_filetype_lcb, added %p to list\n",strarr);
+		gtk_list_store_append(GTK_LIST_STORE(pd->ftd.lstore), &iter);
+		set_filetype_strarr_in_list(&iter,strarr,pd);
+		DEBUG_MSG("add_new_filetype_lcb, set %p to liststore\n",strarr);
+		gtk_tree_selection_select_iter(gtk_tree_view_get_selection(GTK_TREE_VIEW(pd->ftd.lview)),&iter);
 	}
 }
 
@@ -398,26 +419,21 @@ static void filetype_apply_changes(Tprefdialog *pd) {
 				g_strfreev(pd->ftd.curstrarr);
 				tmplist->data = filetype_create_strarr(pd);
 				pd->ftd.curstrarr = tmplist->data;
-				DEBUG_MSG("filetype_apply_changes, new curstrarr=%p\n", pd->ftd.curstrarr);
+				DEBUG_MSG("filetype_apply_changes, applied changes to pd->lists new curstrarr=%p\n", pd->ftd.curstrarr);
 				break;
 			}
 			tmplist = g_list_next(tmplist);
 		}
 		
 		gtk_tree_model_get_iter_first(GTK_TREE_MODEL(pd->ftd.lstore),&iter);
+		DEBUG_MSG("filetype_apply_changes, searching for '%s' in liststore\n",pd->ftd.curstrarr[0]);
 		while (retval) {
 			gchar *curval;
 			gtk_tree_model_get(GTK_TREE_MODEL(pd->ftd.lstore),&iter,0,&curval,-1);
 			if (strcmp(curval,pd->ftd.curstrarr[0])==0) {
-				gchar *escaped = escapestring(pd->ftd.curstrarr[2],'\0');
-				DEBUG_MSG("filetype_apply_changes, set listore 0=%s,1=%s\n", pd->ftd.curstrarr[0], pd->ftd.curstrarr[1]);
-				gtk_list_store_set(GTK_LIST_STORE(pd->ftd.lstore), &iter
-					,0,pd->ftd.curstrarr[0]
-					,1,pd->ftd.curstrarr[1]
-					,2,escaped
-					,3,pd->ftd.curstrarr[3]
-					,-1);
-				g_free(escaped);				
+				DEBUG_MSG("filetype_apply_changes, found it, set listore 0=%s,1=%s,...\n", pd->ftd.curstrarr[0], pd->ftd.curstrarr[1]);
+				set_filetype_strarr_in_list(&iter,pd->ftd.curstrarr,pd);
+				break;
 			}
 			retval = gtk_tree_model_iter_next(GTK_TREE_MODEL(pd->ftd.lstore),&iter);
 		}
@@ -427,8 +443,8 @@ static void filetype_apply_changes(Tprefdialog *pd) {
 }
 
 static void filetype_selection_changed_cb(GtkTreeSelection *selection, Tprefdialog *pd) {
-	generic_selection_changed_cb(selection,pd->ftd.entry,filetype_apply_changes,pd,filetypes,4,&pd->ftd.curstrarr);
-/*	GtkTreeIter iter;
+/*	generic_selection_changed_cb(selection,pd->ftd.entry,filetype_apply_changes,pd,filetypes,4,&pd->ftd.curstrarr);*/
+	GtkTreeIter iter;
 	GtkTreeModel *model;
 	DEBUG_MSG("filetype_selection_changed_cb, started\n");
 	if (gtk_tree_selection_get_selected(selection, &model, &iter)) {
@@ -438,35 +454,39 @@ static void filetype_selection_changed_cb(GtkTreeSelection *selection, Tprefdial
 		filetype_apply_changes(pd);
 		while (tmplist) {
 			gchar **strarr =(gchar **)tmplist->data;
+			DEBUG_MSG("filetype_selection_changed_cb, searching for '%s'\n",filetype);
 			if (strcmp(strarr[0],filetype)==0) {
 				gchar *escaped = escapestring(strarr[2],'\0');
 				gtk_entry_set_text(GTK_ENTRY(pd->ftd.entry[0]), strarr[0]);
 				gtk_entry_set_text(GTK_ENTRY(pd->ftd.entry[1]), strarr[1]);
 				gtk_entry_set_text(GTK_ENTRY(pd->ftd.entry[2]), escaped);
 				gtk_entry_set_text(GTK_ENTRY(pd->ftd.entry[3]), strarr[3]);
+				gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pd->ftd.entry[4]), (strarr[4][0] != '0'));
+				gtk_entry_set_text(GTK_ENTRY(pd->ftd.entry[5]), strarr[5]);
 				pd->ftd.curstrarr = strarr;
 				g_free(escaped);
 				return;
 			}
 			tmplist = g_list_next(tmplist);
 		}
+		DEBUG_MSG("filetype_selection_changed_cb, NOTHING FOUND ?!?!?\n");
 	} else {
 		DEBUG_MSG("filetype_selection_changed_cb, no selection ?!?!\n");
-	}*/
+	}
 }
 
 static void create_filetype_gui(Tprefdialog *pd, GtkWidget *vbox1) {
 	GtkWidget *hbox, *but;
 	pd->lists[filetypes] = duplicate_arraylist(main_v->props.filetypes);
 
-	pd->ftd.lstore = gtk_list_store_new (4,G_TYPE_STRING,G_TYPE_STRING,G_TYPE_STRING,G_TYPE_STRING);
+	pd->ftd.lstore = gtk_list_store_new (6,G_TYPE_STRING,G_TYPE_STRING,G_TYPE_STRING,G_TYPE_STRING,G_TYPE_BOOLEAN,G_TYPE_STRING);
 	pd->ftd.lview = gtk_tree_view_new_with_model(GTK_TREE_MODEL(pd->ftd.lstore));
 	{
 		GtkTreeViewColumn *column;
 		GtkWidget *scrolwin;
 		GtkTreeSelection *select;
-	   GtkCellRenderer *renderer = gtk_cell_renderer_text_new ();
-
+	   GtkCellRenderer *renderer = gtk_cell_renderer_text_new();
+		GtkCellRenderer *renderertoggle = gtk_cell_renderer_toggle_new();
 		column = gtk_tree_view_column_new_with_attributes (_("Filetype"), renderer,"text",0,NULL);
 		gtk_tree_view_append_column (GTK_TREE_VIEW(pd->ftd.lview), column);
 		column = gtk_tree_view_column_new_with_attributes (_("Extensions"), renderer,"text",1,NULL);
@@ -475,6 +495,11 @@ static void create_filetype_gui(Tprefdialog *pd, GtkWidget *vbox1) {
 		gtk_tree_view_append_column (GTK_TREE_VIEW(pd->ftd.lview), column);
 		column = gtk_tree_view_column_new_with_attributes (_("Icon"), renderer,"text",3,NULL);
 		gtk_tree_view_append_column (GTK_TREE_VIEW(pd->ftd.lview), column);
+		
+		column = gtk_tree_view_column_new_with_attributes (_("Editable"), renderertoggle,"active",4,NULL);
+		gtk_tree_view_append_column (GTK_TREE_VIEW(pd->ftd.lview), column);
+		column = gtk_tree_view_column_new_with_attributes (_("Content regex"), renderer,"text",5,NULL);
+		gtk_tree_view_append_column (GTK_TREE_VIEW(pd->ftd.lview), column);
 		scrolwin = gtk_scrolled_window_new(NULL, NULL);
 		gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolwin),GTK_POLICY_AUTOMATIC,GTK_POLICY_AUTOMATIC);
 		gtk_container_add(GTK_CONTAINER(scrolwin), pd->ftd.lview);
@@ -487,18 +512,13 @@ static void create_filetype_gui(Tprefdialog *pd, GtkWidget *vbox1) {
 	{
 		GList *tmplist = g_list_first(pd->lists[filetypes]);
 		while (tmplist) {
+			gint arrcount;
 			gchar **strarr = (gchar **)tmplist->data;
-			if (count_array(strarr)==4) {
+			arrcount = count_array(strarr);
+			if (arrcount==6) {
 				GtkTreeIter iter;
-				gchar *escaped = escapestring(strarr[2],'\0');
 				gtk_list_store_append(GTK_LIST_STORE(pd->ftd.lstore), &iter);
-				gtk_list_store_set(GTK_LIST_STORE(pd->ftd.lstore), &iter
-					,0,strarr[0]
-					,1,strarr[1]
-					,2,escaped
-					,3,strarr[3]
-					,-1);
-				g_free(escaped);
+				set_filetype_strarr_in_list(&iter, strarr,pd);
 			}
 			tmplist = g_list_next(tmplist);
 		}
@@ -516,6 +536,8 @@ static void create_filetype_gui(Tprefdialog *pd, GtkWidget *vbox1) {
 	pd->ftd.entry[1] = boxed_full_entry(_("Extensions (colon separated)"), NULL, 500, vbox1);
 	pd->ftd.entry[2] = boxed_full_entry(_("Highlight update characters"), NULL, 500, vbox1);
 	pd->ftd.entry[3] = prefs_string(_("Icon"), NULL, vbox1, pd, string_file);
+	pd->ftd.entry[4] = boxed_checkbut_with_value(_("Editable"), TRUE, vbox1);
+	pd->ftd.entry[5] = boxed_full_entry(_("Content regex pattern"), NULL, 500, vbox1);
 }
 
 static gchar **filefilter_create_strarr(Tprefdialog *pd) {
@@ -772,7 +794,7 @@ static void highlightpattern_popmenu_activate(GtkMenuItem *menuitem,Tprefdialog
 
 static void add_new_highlightpattern_lcb(GtkWidget *wid, Tprefdialog *pd) {
 	gchar *pattern = gtk_editable_get_chars(GTK_EDITABLE(pd->hpd.entry[0]),0,-1);
-	if (strlen(pattern) && strlen(pd->hpd.selected_filetype)) {
+	if (pattern && pd->hpd.selected_filetype && strlen(pattern) && strlen(pd->hpd.selected_filetype)) {
 		gchar **strarr = highlightpattern_create_strarr(pd);
 		DEBUG_MSG("add_new_highlightpattern_lcb, appending strarr %p to list\n", strarr);
 		pd->lists[highlight_patterns] = g_list_append(pd->lists[highlight_patterns], strarr);
@@ -933,7 +955,7 @@ static void highlightpattern_gui_rebuild_filetype_popup(Tprefdialog *pd) {
 			menuitem = gtk_menu_item_new_with_label(arr[0]);
 			DEBUG_MSG("highlightpattern_gui_rebuild_filetype_popup, menuitem=%p for %s\n", menuitem, arr[0]);
 			g_signal_connect(GTK_OBJECT(menuitem), "activate",G_CALLBACK(highlightpattern_popmenu_activate),pd);
-			gtk_menu_append(GTK_MENU(menu), menuitem);
+			gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
 			gtk_widget_show(menuitem);
 		}
 		tmplist = g_list_next(tmplist);
@@ -1008,10 +1030,10 @@ static void create_highlightpattern_gui(Tprefdialog *pd, GtkWidget *vbox1) {
 	g_signal_connect(G_OBJECT(pd->hpd.radio[1]), "toggled", G_CALLBACK(highlightpattern_type_toggled), pd);
 	g_signal_connect(G_OBJECT(pd->hpd.radio[2]), "toggled", G_CALLBACK(highlightpattern_type_toggled), pd);
 
-	pd->hpd.entry[1] = boxed_full_entry(_("Start pattern"), NULL, 500, vbox3);
-	pd->hpd.entry[2] = boxed_full_entry(_("End pattern"), NULL, 500, vbox3);
+	pd->hpd.entry[1] = boxed_full_entry(_("Start pattern"), NULL, 4000, vbox3);
+	pd->hpd.entry[2] = boxed_full_entry(_("End pattern"), NULL, 4000, vbox3);
 	pd->hpd.check = boxed_checkbut_with_value(_("Case sensitive matching"), FALSE, vbox3);
-	pd->hpd.entry[3] = boxed_full_entry(_("Parentmatch"), NULL, 100, vbox3);
+	pd->hpd.entry[3] = boxed_full_entry(_("Parentmatch"), NULL, 300, vbox3);
 	pd->hpd.entry[4] = prefs_string(_("Foreground color"), "", vbox3, pd, string_color);
 	pd->hpd.entry[5] = prefs_string(_("Background color"), "", vbox3, pd, string_color);
 
@@ -1100,6 +1122,52 @@ static void add_new_browser_lcb(GtkWidget *wid, Tprefdialog *pd) {
 				,-1);
 	}
 }
+
+static void delete_browser_lcb(GtkWidget *wid, Tprefdialog *pd) {
+	if (pd->bd.curstrarr) {
+		GtkTreeIter iter;
+		gboolean retval=TRUE;
+		gtk_tree_model_get_iter_first(GTK_TREE_MODEL(pd->bd.lstore),&iter);
+		while (retval) {
+			gchar *curval;
+			gtk_tree_model_get(GTK_TREE_MODEL(pd->bd.lstore),&iter,0,&curval,-1);
+			if (strcmp(curval,pd->bd.curstrarr[0])==0) {
+				gtk_list_store_remove(GTK_LIST_STORE(pd->bd.lstore),&iter);
+				break;
+			}
+			retval = gtk_tree_model_iter_next(GTK_TREE_MODEL(pd->bd.lstore),&iter);
+		}
+		pd->lists[browsers] = g_list_remove(pd->lists[browsers],pd->bd.curstrarr);
+	}
+}
+
+static void set_default_browser_lcb(GtkWidget *wid, Tprefdialog *pd) {
+	if (pd->bd.curstrarr) {
+		GtkTreeIter iter;
+		gboolean retval=TRUE;
+		gtk_tree_model_get_iter_first(GTK_TREE_MODEL(pd->bd.lstore),&iter);
+		while (retval) {
+			gchar *curval;
+			gtk_tree_model_get(GTK_TREE_MODEL(pd->bd.lstore),&iter,0,&curval,-1);
+			if (strcmp((char *)curval,pd->bd.curstrarr[0])==0) {
+/*				gtk_list_store_remove(GTK_TREE_MODEL(pd->bd.lstore),&iter); */
+                                gtk_list_store_remove((pd->bd.lstore),&iter);
+				break;
+			}
+			retval = gtk_tree_model_iter_next(GTK_TREE_MODEL(pd->bd.lstore),&iter);
+		}
+/*		gtk_list_store_insert(GTK_TREE_MODEL(pd->bd.lstore),&iter,0);*/
+                gtk_list_store_insert((pd->bd.lstore),&iter,0); 
+		gtk_list_store_set(GTK_LIST_STORE(pd->bd.lstore), &iter
+						,0,pd->bd.curstrarr[0]
+						,1,pd->bd.curstrarr[1]
+						,-1);
+		pd->lists[browsers] = g_list_remove(pd->lists[browsers],pd->bd.curstrarr);
+		pd->lists[browsers] = g_list_insert(pd->lists[browsers],pd->bd.curstrarr,0);
+	}
+}
+
+
 static void browser_selection_changed_cb(GtkTreeSelection *selection, Tprefdialog *pd) {
 	DEBUG_MSG("browser_selection_changed_cb, curstrarr=%p, &curstrarr=%p\n", pd->ftd.curstrarr, &pd->bd.curstrarr);
 	generic_selection_changed_cb(selection,pd->bd.entry,browsers_apply_changes,pd,browsers,2,&pd->bd.curstrarr);
@@ -1167,7 +1235,14 @@ static void create_browsers_gui(Tprefdialog *pd, GtkWidget *vbox1) {
 			tmplist = g_list_next(tmplist);
 		}
 	}
+	hbox = gtk_hbox_new(FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(vbox1), hbox, TRUE, TRUE, 0);
+	but = bf_gtkstock_button(GTK_STOCK_DELETE, G_CALLBACK(delete_browser_lcb), pd);
+	gtk_box_pack_start(GTK_BOX(hbox), but, FALSE, TRUE, 3);
+	but = bf_gtkstock_button(GTK_STOCK_GOTO_TOP, G_CALLBACK(set_default_browser_lcb), pd);
+	gtk_box_pack_start(GTK_BOX(hbox), but, FALSE, TRUE, 3);
 	
+	gtk_box_pack_start(GTK_BOX(vbox1), gtk_hseparator_new(), TRUE, TRUE, 0);
 	hbox = gtk_hbox_new(FALSE, 0);
 	gtk_box_pack_start(GTK_BOX(vbox1), hbox, TRUE, TRUE, 0);
 	gtk_box_pack_start(GTK_BOX(hbox), gtk_label_new(_("Browser")), FALSE, TRUE, 3);
@@ -1477,7 +1552,7 @@ static void create_outputbox_gui(Tprefdialog *pd, GtkWidget *vbox1) {
 /* MAIN DIALOG FUNCTIONS              */
 /**************************************/
 
-static void preferences_destroy_lcb(GtkWidget * widget, GdkEvent *event, Tprefdialog *pd) {
+static void preferences_destroy_lcb(GtkWidget * widget, Tprefdialog *pd) {
 	GtkTreeSelection *select;
 	DEBUG_MSG("preferences_destroy_lcb, started\n");
 	select = gtk_tree_view_get_selection(GTK_TREE_VIEW(pd->ftd.lview));
@@ -1501,6 +1576,7 @@ static void preferences_destroy_lcb(GtkWidget * widget, GdkEvent *event, Tprefdi
 static void preferences_ok_clicked_lcb(GtkWidget *wid, Tprefdialog *pd) {
 	string_apply(&main_v->props.editor_font_string, pd->prefs[editor_font_string]);
 	integer_apply(&main_v->props.editor_tab_width, pd->prefs[editor_tab_width], FALSE);
+	integer_apply(&main_v->props.editor_indent_wspaces, pd->prefs[editor_indent_wspaces], TRUE);
 	integer_apply(&main_v->props.word_wrap, pd->prefs[word_wrap], TRUE);
 	integer_apply(&main_v->props.view_line_numbers, pd->prefs[view_line_numbers], TRUE);
 	integer_apply(&main_v->props.defaulthighlight, pd->prefs[defaulthighlight], TRUE);
@@ -1516,10 +1592,16 @@ static void preferences_ok_clicked_lcb(GtkWidget *wid, Tprefdialog *pd) {
 	integer_apply(&main_v->props.auto_set_encoding_meta, pd->prefs[auto_set_encoding_meta], TRUE);
 	integer_apply(&main_v->props.backup_file, pd->prefs[backup_file], TRUE);
 	string_apply(&main_v->props.backup_filestring, pd->prefs[backup_filestring]);
-	string_apply(&main_v->props.backup_abort_style, GTK_COMBO(pd->prefs[backup_abort_style])->entry);
+	main_v->props.backup_abort_action = gtk_option_menu_get_history(GTK_OPTION_MENU(pd->prefs[backup_abort_action]));
+	integer_apply(&main_v->props.backup_cleanuponclose, pd->prefs[backup_cleanuponclose], TRUE);
 	integer_apply(&main_v->props.num_undo_levels, pd->prefs[num_undo_levels], FALSE);
 	integer_apply(&main_v->props.clear_undo_on_save, pd->prefs[clear_undo_on_save], TRUE);
 	integer_apply(&main_v->props.allow_multi_instances, pd->prefs[allow_multi_instances], TRUE);
+#ifdef WITH_MSG_QUEUE
+	integer_apply(&main_v->props.open_in_running_bluefish, pd->prefs[open_in_running_bluefish], TRUE);
+#endif
+	integer_apply(&main_v->props.max_recent_files, pd->prefs[max_recent_files], FALSE);
+	
 	integer_apply(&main_v->props.restore_dimensions, pd->prefs[restore_dimensions], TRUE);
 	if (!main_v->props.restore_dimensions) {
 		integer_apply(&main_v->props.left_panel_width, pd->prefs[left_panel_width], FALSE);
@@ -1527,6 +1609,9 @@ static void preferences_ok_clicked_lcb(GtkWidget *wid, Tprefdialog *pd) {
 		integer_apply(&main_v->props.main_window_w, pd->prefs[main_window_w], FALSE);
 	}
 	string_apply(&main_v->props.tab_font_string, pd->prefs[tab_font_string]);
+	main_v->props.document_tabposition = gtk_option_menu_get_history(GTK_OPTION_MENU(pd->prefs[document_tabposition]));
+	main_v->props.leftpanel_tabposition = gtk_option_menu_get_history(GTK_OPTION_MENU(pd->prefs[leftpanel_tabposition]));
+	
 	integer_apply(&main_v->props.ext_browsers_in_submenu, pd->prefs[ext_browsers_in_submenu], TRUE);
 	integer_apply(&main_v->props.ext_commands_in_submenu, pd->prefs[ext_commands_in_submenu], TRUE);
 	integer_apply(&main_v->props.ext_outputbox_in_submenu, pd->prefs[ext_outputbox_in_submenu], TRUE);
@@ -1579,8 +1664,9 @@ static void preferences_ok_clicked_lcb(GtkWidget *wid, Tprefdialog *pd) {
 	menu_outputbox_rebuild();
 	encoding_menu_rebuild();
 	external_menu_rebuild(); /* browsers is also rebuild here! */
-	
-	preferences_destroy_lcb(NULL, NULL, pd);
+	gui_apply_settings();
+	all_documents_apply_settings();
+	preferences_destroy_lcb(NULL, pd);
 }
 static void preferences_cancel_clicked_lcb(GtkWidget *wid, Tprefdialog *pd) {
 	DEBUG_MSG("preferences_cancel_clicked_lcb, started\n");
@@ -1594,7 +1680,7 @@ static void preferences_cancel_clicked_lcb(GtkWidget *wid, Tprefdialog *pd) {
 	pd->lists[highlight_patterns] = NULL;
 	pd->lists[browsers] = NULL;
 	pd->lists[external_commands] = NULL;
-	preferences_destroy_lcb(NULL, NULL, pd);
+	preferences_destroy_lcb(NULL, pd);
 }
 
 static void restore_dimensions_toggled_lcb(GtkToggleButton *togglebutton,Tprefdialog *pd) {
@@ -1604,12 +1690,13 @@ static void restore_dimensions_toggled_lcb(GtkToggleButton *togglebutton,Tprefdi
 }
 static void create_backup_toggled_lcb(GtkToggleButton *togglebutton,Tprefdialog *pd) {
 	gtk_widget_set_sensitive(pd->prefs[backup_filestring], togglebutton->active);
-	gtk_widget_set_sensitive(pd->prefs[backup_abort_style], togglebutton->active);
+	gtk_widget_set_sensitive(pd->prefs[backup_abort_action], togglebutton->active);
 }
 
 static void preferences_dialog() {
 	Tprefdialog *pd;
 	GtkWidget *dvbox, *frame, *vbox1, *vbox2;
+	gchar *notebooktabpositions[] = {N_("left"), N_("right"), N_("top"), N_("bottom"), NULL};
 
 	pd = g_new0(Tprefdialog,1);
 	pd->win = window_full(_("Edit preferences"), GTK_WIN_POS_NONE, 0, G_CALLBACK(preferences_destroy_lcb), pd, TRUE);
@@ -1632,6 +1719,7 @@ static void preferences_dialog() {
 	
 	pd->prefs[editor_font_string] = prefs_string(_("Font"), main_v->props.editor_font_string, vbox2, pd, string_font);
 	pd->prefs[editor_tab_width] = prefs_integer(_("Tab width"), main_v->props.editor_tab_width, vbox2, pd, 1, 50);
+	pd->prefs[editor_indent_wspaces] = boxed_checkbut_with_value(_("Use spaces to indent, not tabs"), main_v->props.editor_indent_wspaces, vbox2);
 	pd->prefs[word_wrap] = boxed_checkbut_with_value(_("Word wrap default"), main_v->props.word_wrap, vbox2);
 	pd->prefs[view_line_numbers] = boxed_checkbut_with_value(_("Line numbers by default"), main_v->props.view_line_numbers, vbox2);
 	pd->prefs[defaulthighlight] = boxed_checkbut_with_value(_("Highlight syntax by default"), main_v->props.defaulthighlight, vbox2);
@@ -1677,12 +1765,10 @@ static void preferences_dialog() {
 	pd->prefs[backup_file] = boxed_checkbut_with_value(_("Create backup on save"), main_v->props.backup_file, vbox2);
 	pd->prefs[backup_filestring] = prefs_string(_("Backup file suffix"), main_v->props.backup_filestring, vbox2, pd, string_none);
 	{
-		GList *poplist = g_list_append(NULL, "save");
-		poplist = g_list_append(poplist, "abort");
-		poplist = g_list_append(poplist, "ask");
-		pd->prefs[backup_abort_style] = prefs_combo(_("Action on backup failure"),main_v->props.backup_abort_style, vbox2, pd, poplist, FALSE);
-		g_list_free(poplist);
+		gchar *failureactions[] = {N_("save"), N_("abort"), N_("ask"), NULL};
+		pd->prefs[backup_abort_action] = boxed_optionmenu_with_value(_("Action on backup failure"), main_v->props.backup_abort_action, vbox2, failureactions);
 	}
+	pd->prefs[backup_cleanuponclose] = boxed_checkbut_with_value(_("Remove backupfile on close"), main_v->props.backup_cleanuponclose, vbox2);
 	create_backup_toggled_lcb(GTK_TOGGLE_BUTTON(pd->prefs[backup_file]), pd);
 	g_signal_connect(G_OBJECT(pd->prefs[backup_file]), "toggled", G_CALLBACK(create_backup_toggled_lcb), pd);
 
@@ -1700,6 +1786,10 @@ static void preferences_dialog() {
 	gtk_container_add(GTK_CONTAINER(frame), vbox2);
 
 	pd->prefs[allow_multi_instances] = boxed_checkbut_with_value(_("Allow multi instances of a file"), main_v->props.allow_multi_instances, vbox2);
+#ifdef WITH_MSG_QUEUE
+	pd->prefs[open_in_running_bluefish] = boxed_checkbut_with_value(_("Open files in already running bluefish window"),main_v->props.open_in_running_bluefish, vbox2);
+#endif /* WITH_MSG_QUEUE */		
+	pd->prefs[max_recent_files] = prefs_integer(_("Number of files in 'Open recent'"), main_v->props.max_recent_files, vbox2, pd, 3, 100);
 
 	vbox1 = gtk_vbox_new(FALSE, 5);
 	gtk_notebook_append_page(GTK_NOTEBOOK(pd->noteb), vbox1, hbox_with_pix_and_text(_("User interface"), 156));
@@ -1725,6 +1815,10 @@ static void preferences_dialog() {
 	pd->prefs[ext_outputbox_in_submenu] = boxed_checkbut_with_value(_("External outputbox in submenu"), main_v->props.ext_outputbox_in_submenu, vbox2);
 	
 	pd->prefs[tab_font_string] = prefs_string(_("Notebook tab font (leave empty for gtk default)"), main_v->props.tab_font_string, vbox2, pd, string_font);
+	
+	pd->prefs[document_tabposition] = boxed_optionmenu_with_value(_("Document notebook tab position"), main_v->props.document_tabposition, vbox2, notebooktabpositions);
+	pd->prefs[leftpanel_tabposition] = boxed_optionmenu_with_value(_("Left panel notebook tab position"), main_v->props.leftpanel_tabposition, vbox2, notebooktabpositions);
+	
 	pd->prefs[transient_htdialogs] = boxed_checkbut_with_value(_("Make HTML dialogs transient"), main_v->props.transient_htdialogs, vbox2);
 	pd->prefs[default_advanced_snr] = boxed_checkbut_with_value(_("Advanced search and replace by default"), main_v->props.default_advanced_snr, vbox2);
 
