@@ -28,57 +28,39 @@
 #include <string.h>
 #include <stdlib.h>
 #include <stdio.h>
+#include <time.h>
 
 #include "bluefish.h"
 #include "rcfile.h"
 #include "bf_lib.h"
+#include "fref.h"
 #include "stringlist.h"
 #include "highlight.h" /* hl_reset_to_default()*/
+#include "document.h" /* DOCUMENT_BACKUP_ABORT_ASK */
 
 typedef struct {
-	void *pointer;
-	unsigned char type;
-	gchar *identifier;
+	void *pointer; /* where should the value be stored ?*/
+	unsigned char type; /* a=arraylist, l=stringlist, s=string, e=string with escape, i=integer */
+	gchar *identifier; /* the string that should be in the config file for this entry */
+	gint len; /* only used for arrays, the length the array should have */
 } Tconfig_list_item;
 
 static GList *main_configlist=NULL;
 static GList *highlighting_configlist=NULL;
 static GList *custom_menu_configlist=NULL;
 
-gchar **array_from_arglist(const gchar *string1, ...) {
-	gint numargs=1;
-	va_list args;
-	gchar *s;
-	gchar **retval, **index;
-
-	va_start (args, string1);
-	s = va_arg (args, gchar*);
-	while (s) {
-		numargs++;
-		s = va_arg (args, gchar*);
+static void free_configlist(GList *configlist) {
+	GList *tmplist = g_list_first(configlist);
+	while(tmplist) {
+		Tconfig_list_item *cli = tmplist->data;
+		g_free(cli);
+		tmplist = g_list_next(tmplist);
 	}
-	va_end (args);
-	DEBUG_MSG("array_from_arglist, numargs=%d\n", numargs);
-
-	index = retval = g_new(gchar *, numargs + 1);
-	*index = g_strdup(string1);
-
-	va_start (args, string1);
-	s = va_arg (args, gchar*);
-	while (s) {
-		index++;
-		*index = g_strdup(s);
-		s = va_arg (args, gchar*);
-	}
-	va_end (args);
-	index++;
-	*index = NULL;
-	return retval;
+	g_list_free(configlist);
 }
 
-
 /*this should add 1 empty entry to the configuration list */
-GList *make_config_list_item(GList * config_list, void *pointer_to_var, unsigned char type_of_var, gchar * name_of_var)
+GList *make_config_list_item(GList * config_list, void *pointer_to_var, unsigned char type_of_var, gchar * name_of_var, gint len)
 {
 	Tconfig_list_item *config_list_item;
 	if (!pointer_to_var) {
@@ -89,39 +71,40 @@ GList *make_config_list_item(GList * config_list, void *pointer_to_var, unsigned
 	config_list_item->pointer = pointer_to_var;
 	config_list_item->type = type_of_var;
 	config_list_item->identifier = name_of_var;
+	config_list_item->len = len;
 	return (GList *) g_list_append(config_list, config_list_item);
 }
 
 static void init_prop_integer(GList ** config_list, void *pointer_to_var, gchar * name_of_var, gint default_value)
 {
-	*config_list = make_config_list_item(*config_list, pointer_to_var, 'i', name_of_var);
+	*config_list = make_config_list_item(*config_list, pointer_to_var, 'i', name_of_var, 0);
 	*(gint *) pointer_to_var = default_value;
 }
 
 static void init_prop_string(GList ** config_list, void *pointer_to_var, gchar * name_of_var, gchar * default_value)
 {
-	*config_list = make_config_list_item(*config_list, pointer_to_var, 's', name_of_var);
+	*config_list = make_config_list_item(*config_list, pointer_to_var, 's', name_of_var, 0);
 	*(gchar **) pointer_to_var = g_strdup(default_value);
 	DEBUG_MSG("init_prop_string, name_of_var=%s, default_value=%s\n", name_of_var, default_value);
 }
 
 static void init_prop_string_with_escape(GList ** config_list, void *pointer_to_var, gchar * name_of_var, gchar * default_value)
 {
-	*config_list = make_config_list_item(*config_list, pointer_to_var, 'e', name_of_var);
+	*config_list = make_config_list_item(*config_list, pointer_to_var, 'e', name_of_var, 0);
 	*(gchar **) pointer_to_var = unescapestring(default_value);
 	DEBUG_MSG("init_prop_string, name_of_var=%s, default_value=%s\n", name_of_var, default_value);
 }
 
 static void init_prop_stringlist(GList ** config_list, void *pointer_to_var, gchar * name_of_var)
 {
-	*config_list = make_config_list_item(*config_list, pointer_to_var, 'l', name_of_var);
+	*config_list = make_config_list_item(*config_list, pointer_to_var, 'l', name_of_var, 0);
 	pointer_to_var = NULL;
 }
 
 
-static void init_prop_arraylist(GList ** config_list, void *pointer_to_var, gchar * name_of_var)
+static void init_prop_arraylist(GList ** config_list, void *pointer_to_var, gchar * name_of_var, gint len)
 {
-	*config_list = make_config_list_item(*config_list, pointer_to_var, 'a', name_of_var);
+	*config_list = make_config_list_item(*config_list, pointer_to_var, 'a', name_of_var, len);
 	pointer_to_var = NULL;
 }
 
@@ -292,7 +275,12 @@ static gboolean parse_config_file(GList * config_list, gchar * filename)
 						break;
 					case 'a':
 						tmparray = string_to_array(tmpstring, ':');
-						* (void **) tmpitem->pointer = g_list_prepend((GList *) * (void **) tmpitem->pointer, tmparray);
+						if (tmpitem->len <= 0 || tmpitem->len == count_array(tmparray)) {
+							* (void **) tmpitem->pointer = g_list_prepend((GList *) * (void **) tmpitem->pointer, tmparray);
+						} else {
+							DEBUG_MSG("parse_config_file, not storing array, count_array() != tmpitem->len\n");
+							g_strfreev(tmparray);
+						}
 						DEBUG_MSG("parse_config_file, *(void **)tmpitem->pointer=%p\n", *(void **) tmpitem->pointer);
 						break;
 					default:
@@ -318,21 +306,23 @@ static GList *props_init_main(GList * config_rc)
 	init_prop_integer   (&config_rc, &main_v->props.view_main_toolbar, "view_main_toolbar:", 1);
 	init_prop_integer   (&config_rc, &main_v->props.view_left_panel, "view_left_panel:", 1);
 	init_prop_integer   (&config_rc, &main_v->props.view_line_numbers, "view_line_numbers:", 1);
-	init_prop_integer   (&config_rc, &main_v->props.filebrowser_show_hidden_files, "fb_show_hidden_f:", 0);
-	init_prop_integer   (&config_rc, &main_v->props.filebrowser_show_others_files, "fb_show_others_f:", 0);
+	init_prop_integer   (&config_rc, &main_v->props.filebrowser_show_hidden_files, "fb_show_hidden_f:", 1);
+	init_prop_integer   (&config_rc, &main_v->props.filebrowser_show_others_files, "fb_show_others_f:", 1);
 	init_prop_integer   (&config_rc, &main_v->props.filebrowser_show_backup_files, "fb_show_backup_f:", 0);
 	init_prop_string    (&config_rc, &main_v->props.filebrowser_unknown_icon, "fb_unknown_icon:", PKGDATADIR"icon_unknown.png");
 	init_prop_string    (&config_rc, &main_v->props.filebrowser_dir_icon, "fb_dir_icon:", PKGDATADIR"icon_dir.png");
 	init_prop_string    (&config_rc, &main_v->props.editor_font_string, "editor_font_string:", "courier 11");
 	init_prop_integer   (&config_rc, &main_v->props.editor_tab_width, "editor_tab_width:", 3);
 	init_prop_string    (&config_rc, &main_v->props.tab_font_string, "tab_font_string:", "");
-	init_prop_arraylist (&config_rc, &main_v->props.browsers, "browsers:");
-	init_prop_arraylist (&config_rc, &main_v->props.external_commands, "external_commands:");
+	init_prop_arraylist (&config_rc, &main_v->props.browsers, "browsers:", 2);
+	init_prop_arraylist (&config_rc, &main_v->props.external_commands, "external_commands:", 2);
 	init_prop_stringlist(&config_rc, &main_v->props.quickbar_items, "quickbar_items:");
 	init_prop_integer   (&config_rc, &main_v->props.highlight_num_lines_count, "highlight_num_lines_count:", 1);
 	init_prop_integer   (&config_rc, &main_v->props.defaulthighlight, "defaulthighlight:", 1);
-	init_prop_arraylist (&config_rc, &main_v->props.filetypes, "filetypes:");
-	init_prop_arraylist (&config_rc, &main_v->props.filefilters, "filefilters:");
+	/* old type filetypes have a different count, they are converted below */
+	init_prop_arraylist (&config_rc, &main_v->props.filetypes, "filetypes:", 0);
+	init_prop_integer   (&config_rc, &main_v->props.numcharsforfiletype, "numcharsforfiletype:", 200);
+	init_prop_arraylist (&config_rc, &main_v->props.filefilters, "filefilters:", 3);
 	init_prop_string    (&config_rc, &main_v->props.last_filefilter, "last_filefilter:", "");
 	init_prop_integer   (&config_rc, &main_v->props.transient_htdialogs, "transient_htdialogs:", 1);
 	init_prop_integer   (&config_rc, &main_v->props.restore_dimensions, "restore_dimensions:", 1);	
@@ -343,7 +333,8 @@ static GList *props_init_main(GList * config_rc)
 	init_prop_integer   (&config_rc, &main_v->props.max_dir_history, "max_dir_history:", 10);
 	init_prop_integer   (&config_rc, &main_v->props.backup_file,"backup_file:",1);
 	init_prop_string    (&config_rc, &main_v->props.backup_filestring,"backup_filestring:","~");
-	init_prop_string    (&config_rc, &main_v->props.backup_abort_style,"backup_abort_style:","ask");
+	init_prop_integer    (&config_rc, &main_v->props.backup_abort_action,"backup_abort_action:",DOCUMENT_BACKUP_ABORT_ASK);
+	init_prop_integer   (&config_rc, &main_v->props.backup_cleanuponclose,"backup_cleanuponclose:",0);
 	init_prop_string    (&config_rc, &main_v->props.image_thumbnailstring, "image_thumbnailstring:", "_thumbnail");
 	init_prop_string    (&config_rc, &main_v->props.image_thumbnailtype, "image_thumbnailtype:", "png");
 	init_prop_integer   (&config_rc, &main_v->props.image_thumbnail_refresh_quality,"image_thumbnail_refresh_quality:",1);
@@ -355,23 +346,30 @@ static GList *props_init_main(GList * config_rc)
 	init_prop_integer   (&config_rc, &main_v->props.num_undo_levels,"num_undo_levels:",100);
 	init_prop_integer   (&config_rc, &main_v->props.clear_undo_on_save,"clear_undo_on_save:",0);
 	init_prop_string    (&config_rc, &main_v->props.newfile_default_encoding,"newfile_default_encoding:","UTF-8");
-	init_prop_arraylist (&config_rc, &main_v->props.encodings, "encodings:");
+	init_prop_arraylist (&config_rc, &main_v->props.encodings, "encodings:", 2);
 	init_prop_integer   (&config_rc, &main_v->props.auto_set_encoding_meta,"auto_set_encoding_meta:",1);
 	init_prop_integer   (&config_rc, &main_v->props.auto_update_meta,"auto_update_meta:",1);
-	init_prop_arraylist (&config_rc, &main_v->props.outputbox, "outputbox:");
+	init_prop_arraylist (&config_rc, &main_v->props.outputbox, "outputbox:", 7);
 	init_prop_integer   (&config_rc, &main_v->props.ext_browsers_in_submenu,"ext_browsers_in_submenu:",0);
 	init_prop_integer   (&config_rc, &main_v->props.ext_commands_in_submenu,"ext_commands_in_submenu:",1);
 	init_prop_integer   (&config_rc, &main_v->props.ext_outputbox_in_submenu,"ext_outputbox_in_submenu:",1);
+	init_prop_arraylist (&config_rc, &main_v->props.reference_files, "reference_files:", 2);
+	init_prop_integer   (&config_rc, &main_v->props.fref_ldoubleclick_action,"fref_ldoubleclick_action:",0);
+	init_prop_integer   (&config_rc, &main_v->props.fref_info_type,"fref_info_type:",0);	
+	init_prop_integer   (&config_rc, &main_v->props.document_tabposition,"document_tabposition:",(gint)GTK_POS_BOTTOM);
+	init_prop_integer   (&config_rc, &main_v->props.leftpanel_tabposition,"leftpanel_tabposition:",(gint)GTK_POS_BOTTOM);
 #ifdef HAVE_LIBASPELL
 	init_prop_string(&config_rc, &main_v->props.spell_default_lang, "spell_default_lang:", "en");
 #endif /* HAVE_LIBASPELL */
 	init_prop_integer   (&config_rc, &main_v->props.default_advanced_snr,"default_advanced_snr:",0);
 	init_prop_integer(&config_rc, &main_v->props.cont_highlight_full, "cont_highlight_full:", 1);
+	init_prop_integer(&config_rc, &main_v->props.lasttime_cust_menu, "lasttime_cust_menu:", 0);
+	init_prop_integer(&config_rc, &main_v->props.lasttime_highlighting, "lasttime_highlighting:", 0);
+	init_prop_integer(&config_rc, &main_v->props.lasttime_filetypes, "lasttime_filetypes:", 0);
+	init_prop_integer(&config_rc, &main_v->props.lasttime_encodings, "lasttime_encodings:", 0);
 
 	/* not yet in use */
 	init_prop_string(&config_rc, &main_v->props.image_editor_cline, "image_editor_command:", "gimp-remote -n \"%s\"&");
-	init_prop_string(&config_rc, &main_v->props.cfg_tab_pos, "notebook_tab_position:", "bottom");
-/*	init_prop_string(&config_rc, &main_v->props.cfg_weblint_cline, "weblint_command:", WEBLINT_COMMAND);*/
 	init_prop_integer(&config_rc, &main_v->props.full_p, "closing_paragraph_tag:", 1);
 	init_prop_integer(&config_rc, &main_v->props.full_li, "closing_list_item_tag:", 1);
 	init_prop_integer(&config_rc, &main_v->props.allow_css, "allow_the_use_of_css:", 1);
@@ -407,6 +405,19 @@ static GList *props_init_main(GList * config_rc)
 	return config_rc;
 }
 
+/* we save the value in 'days' precision, so we can divide seconds by 24*60*60
+, this way we can store it in a gint (which is the config file precision) */
+#define TIME_T_TO_GINT(time) ((gint)(time / (24*60*60)))
+
+static gboolean config_file_is_newer(gint lasttime, const gchar *configfile) {
+	struct stat statbuf;
+	if(stat(configfile, &statbuf)==0) {
+		if (TIME_T_TO_GINT(statbuf.st_mtime) >= lasttime) return TRUE;
+	}
+	return FALSE;
+}
+
+/*
 static GList *arraylist_load_defaults(GList *thelist, const gchar *filename, const gchar *name) {
 	GList *deflist,*tmplist = g_list_first(thelist);
 	if (name) {
@@ -454,7 +465,7 @@ static GList *arraylist_load_defaults(GList *thelist, const gchar *filename, con
 	}
 	return thelist;
 }
-
+*/
 void rcfile_parse_main(void)
 {
 	gchar *filename;
@@ -487,43 +498,59 @@ void rcfile_parse_main(void)
 		arr = array_from_arglist(_("Gnome default"), "gnome-moz-remote --newwin %s&",NULL);
 		main_v->props.browsers = g_list_append(main_v->props.browsers,arr);
 	}
-	if (main_v->props.encodings == NULL) {
-		/* if the user does not have encodings --> set them to defaults values */
-		gchar *filename = return_first_existing_filename(PKGDATADIR"encodings.default",
-										"data/encodings.default",
-									"../data/encodings.default",NULL);
-		if (filename) {
-			main_v->props.encodings = arraylist_load_defaults(main_v->props.encodings,filename,NULL);
+	{
+		gchar *defaultfile = return_first_existing_filename(PKGDATADIR"encodings.default",
+											"data/encodings.default",
+										"../data/encodings.default",NULL);
+		if (main_v->props.encodings == NULL) {
+			/* if the user does not have encodings --> set them to defaults values */
+			if (defaultfile) {
+				main_v->props.encodings = get_list(defaultfile,NULL,TRUE);
+			} else {
+				g_print("Unable to find '"PKGDATADIR"encodings.default'\n");
+			}
 		} else {
-			g_print("Unable to find '"PKGDATADIR"encodings.default'\n");
+			if (config_file_is_newer(main_v->props.lasttime_encodings,defaultfile)) {
+				main_v->props.encodings = arraylist_load_new_identifiers_from_file(main_v->props.encodings,defaultfile,1);
+				main_v->props.lasttime_encodings = TIME_T_TO_GINT(time(NULL));
+			}
 		}
+		g_free(defaultfile);
 	}
 	if (main_v->props.outputbox==NULL) {
 		/* if the user does not have outputbox settings --> set them to defaults values */
 		main_v->props.outputbox = g_list_append(main_v->props.outputbox,array_from_arglist(_("make"),"([a-zA-Z0-9/_.-]+):([0-9]+):(.*)","1","2","3","make","1",NULL));
-		main_v->props.outputbox = g_list_append(main_v->props.outputbox,array_from_arglist(_("weblint HTML checker"),"([a-zA-Z0-9/_.-]+)\\(([0-9]+)\\): (.*)","1","2","3","weblint %s","1",NULL));
-		main_v->props.outputbox = g_list_append(main_v->props.outputbox,array_from_arglist(_("tidy HTML validator"),"line ([0-9]+) column [0-9]+ - (.*)","-1","1","2","tidy -qe %s","0",NULL));
-		main_v->props.outputbox = g_list_append(main_v->props.outputbox,array_from_arglist(_("javac"),"([a-zA-Z0-9/_.-]+):([0-9]+):(.*)","1","2","3","javac %s","0",NULL));
+		main_v->props.outputbox = g_list_append(main_v->props.outputbox,array_from_arglist(_("weblint HTML checker"),"([a-zA-Z0-9/_.-]+)\\(([0-9]+)\\): (.*)","1","2","3","weblint '%s'","1",NULL));
+		main_v->props.outputbox = g_list_append(main_v->props.outputbox,array_from_arglist(_("tidy HTML validator"),"line ([0-9]+) column [0-9]+ - (.*)","-1","1","2","tidy -qe '%s'","0",NULL));
+		main_v->props.outputbox = g_list_append(main_v->props.outputbox,array_from_arglist(_("javac"),"([a-zA-Z0-9/_.-]+):([0-9]+):(.*)","1","2","3","javac '%s'","0",NULL));
 /*		main_v->props.outputbox = g_list_append(main_v->props.outputbox,array_from_arglist(,NULL)); */
 	}
 	if (main_v->props.external_commands == NULL) {
 		/* if the user does not have external commands --> set them to defaults values */
 		gchar **arr;
-		arr = array_from_arglist(_("Dos2Unix filter"), "cat %s | dos2unix > %f",NULL);
+		arr = array_from_arglist(_("Dos2Unix filter"), "cat '%s' | dos2unix > '%f'",NULL);
 		main_v->props.external_commands = g_list_append(main_v->props.external_commands,arr);
-		arr = array_from_arglist(_("Tidy cleanup filter"), "cat %s | tidy -utf8 -q >%f 2>/dev/null",NULL);
+		arr = array_from_arglist(_("Tidy cleanup filter"), "cat '%s' | tidy -utf8 -q >'%f' 2>/dev/null",NULL);
 		main_v->props.external_commands = g_list_append(main_v->props.external_commands,arr);
 	}
-	if (main_v->props.filetypes == NULL) {
-		/* if the user does not have file-types --> set them to defaults values */
-		gchar *filename = return_first_existing_filename(PKGDATADIR"filetypes.default",
+	{
+	gchar *defaultfile = return_first_existing_filename(PKGDATADIR"filetypes.default",
 										"data/filetypes.default",
 									"../data/filetypes.default",NULL);
-		if (filename) {
-			main_v->props.filetypes = arraylist_load_defaults(main_v->props.filetypes,filename,NULL);
+		if (main_v->props.filetypes == NULL) {
+			/* if the user does not have file-types --> set them to defaults values */
+			if (defaultfile) {
+				main_v->props.filetypes = get_list(defaultfile,NULL,TRUE);
+			} else {
+				g_print("Unable to find '"PKGDATADIR"filetypes.default'\n");
+			}
 		} else {
-			g_print("Unable to find '"PKGDATADIR"filetypes.default'\n");
+			if (config_file_is_newer(main_v->props.lasttime_filetypes,defaultfile)) {
+				main_v->props.filetypes = arraylist_load_new_identifiers_from_file(main_v->props.filetypes,defaultfile,1);
+				main_v->props.lasttime_filetypes = TIME_T_TO_GINT(time(NULL));
+			}
 		}
+		g_free(defaultfile);
 	}
 	if (main_v->props.filefilters == NULL) {
 		/* if the user does not have file filters --> set them to defaults values */
@@ -539,24 +566,33 @@ void rcfile_parse_main(void)
 		arr = array_from_arglist(_("Hide objectfiles"),"0", "objectfile", NULL);
 		main_v->props.filefilters = g_list_append(main_v->props.filefilters, arr);
 	}
+	if (main_v->props.reference_files == NULL) {
+		gchar *userdir = g_strconcat(g_get_home_dir(), "/.bluefish/", NULL);
+		/* if the user does not yet have any function reference files, set them to default values */
+		fref_rescan_dir(PKGDATADIR);
+		fref_rescan_dir(userdir);
+		g_free(userdir);
+	}
+	/* for backwards compatibility with old filetypes */
+	{
+		GList *tmplist = g_list_first(main_v->props.filetypes);
+		while (tmplist) {
+			gchar **orig = (gchar **)tmplist->data;
+			if (count_array(orig)==4) {
+				gchar **new = array_from_arglist(orig[0], orig[1], orig[2], orig[3], "1", "", NULL);
+				tmplist->data = new;
+				g_strfreev(orig);
+			}
+			tmplist = g_list_next(tmplist);
+		}
+	}
 }
 
 static gint rcfile_save_main(void) {
 	gchar *filename = g_strconcat(g_get_home_dir(), "/.bluefish/rcfile_v2", NULL);
 	return save_config_file(main_configlist, filename);
 }
-
-static void load_default_highlightingpatterns(gchar *name) {
-	gchar *filename = return_first_existing_filename(PKGDATADIR"highlighting.default",
-									"data/highlighting.default",
-									"../data/highlighting.default",NULL);
-	if (filename) {
-		main_v->props.highlight_patterns = arraylist_load_defaults(main_v->props.highlight_patterns,filename,name);
-	} else {
-		g_print("Unable to find '"PKGDATADIR"highlighting.default'\n");
-	}
-}
-
+/*
 static gboolean arraylist_test_identifier_exists(GList *arrlist, const gchar *name) {
 	GList *tmplist = g_list_first(arrlist);
 	while(tmplist) {
@@ -567,38 +603,39 @@ static gboolean arraylist_test_identifier_exists(GList *arrlist, const gchar *na
 	}
 	return FALSE;
 }
-
+*/
 void rcfile_parse_highlighting(void) {
 	gchar *filename;
+	gchar *defaultfile;
 
 	DEBUG_MSG("rcfile_parse_highlighting, started\n");
 
 	highlighting_configlist = NULL;
-	init_prop_arraylist(&highlighting_configlist, &main_v->props.highlight_patterns, "patterns:");
+	init_prop_arraylist(&highlighting_configlist, &main_v->props.highlight_patterns, "patterns:", 0);
 
 	filename = g_strconcat(g_get_home_dir(), "/.bluefish/highlighting", NULL);
+	defaultfile = return_first_existing_filename(PKGDATADIR"highlighting.default",
+									"data/highlighting.default",
+									"../data/highlighting.default",NULL);
 	if (!parse_config_file(highlighting_configlist, filename)) {
 		/* init the highlighting in some way? */
-		load_default_highlightingpatterns(NULL);
+		if (defaultfile) {
+			main_v->props.highlight_patterns = get_list(defaultfile,NULL,TRUE);
+		} else {
+			g_print("Unable to find '"PKGDATADIR"highlighting.default'\n");
+		}
 		save_config_file(highlighting_configlist, filename);
 		DEBUG_MSG("rcfile_parse_highlighting, done saving\n");
 	} else {
-		gchar *types[] = {"php", "html", "c", "java", "xml","cfml","python", NULL};
-		gint i = 0;
-		while (types[i]) {
-			DEBUG_MSG("rcfile_parse_highlighting, testing for type %s\n", types[i]);
-			if (!arraylist_test_identifier_exists(main_v->props.highlight_patterns,types[i])) {
-				DEBUG_MSG("rcfile_parse_highlighting, type %s not found, reloading from default\n", types[i]);
-				load_default_highlightingpatterns(types[i]);
-			}
-			i++;
+		if (config_file_is_newer(main_v->props.lasttime_highlighting,defaultfile)) {
+			main_v->props.highlight_patterns = arraylist_load_new_identifiers_from_file(main_v->props.highlight_patterns,defaultfile,2);
+			main_v->props.lasttime_highlighting = TIME_T_TO_GINT(time(NULL));
 		}
 	}
 	g_free(filename);
+	g_free(defaultfile);
 }
 
-
-
 static gint rcfile_save_highlighting(void) {
 	gint retval;
 	gchar *filename = g_strconcat(g_get_home_dir(), "/.bluefish/highlighting", NULL);
@@ -606,18 +643,85 @@ static gint rcfile_save_highlighting(void) {
 	g_free(filename);
 	return retval;
 }
+
 void rcfile_parse_custom_menu(void) {
 	gchar *filename;
-
+	gchar *defaultfile;
+	GList *cust_menu = NULL;
 	DEBUG_MSG("rcfile_parse_custom_menu, started\n");
 
 	custom_menu_configlist = NULL;
-	init_prop_arraylist(&custom_menu_configlist, &main_v->props.cust_menu, "custom_menu:");
+	init_prop_arraylist(&custom_menu_configlist, &cust_menu, "custom_menu:", 0);
+	init_prop_arraylist(&custom_menu_configlist, &main_v->props.cmenu_insert, "cmenu_insert:", 0);
+	init_prop_arraylist(&custom_menu_configlist, &main_v->props.cmenu_replace, "cmenu_replace:", 0);
 
 	filename = g_strconcat(g_get_home_dir(), "/.bluefish/custom_menu", NULL);
-	if (!parse_config_file(custom_menu_configlist, filename) || main_v->props.cust_menu==NULL) {
+	defaultfile = return_first_existing_filename(PKGDATADIR"custom_menu.default",
+									"data/custom_menu.default",
+									"../data/custom_menu.default",NULL);
+	if (!parse_config_file(custom_menu_configlist, filename) || (main_v->props.cust_menu==NULL && main_v->props.cmenu_insert==NULL && main_v->props.cmenu_replace==NULL )) {
 		/* init the custom_menu in some way? */
-		main_v->props.cust_menu = g_list_append(main_v->props.cust_menu, array_from_arglist(N_("/php/ibase/ibase fetch row"), "0", "while ($%1 = ibase_fetch_row($%0)) {\n	echo $%1[0];\n", "\n}\nibase_free_result($%0);\n", "2", _("ibase result variable name"), _("row variable name"), NULL));
+		if (defaultfile) {
+			parse_config_file(custom_menu_configlist, defaultfile);
+		} else {
+			g_print("Unable to find '"PKGDATADIR"custom_menu.default'\n");
+		}
+	} else {
+		if (config_file_is_newer(main_v->props.lasttime_cust_menu,defaultfile)) {
+			GList *default_insert=NULL, *default_replace=NULL, *tmp_configlist=NULL;
+			init_prop_arraylist(&tmp_configlist, &default_insert, "cmenu_insert:", 0);
+			init_prop_arraylist(&tmp_configlist, &default_replace, "cmenu_replace:", 0);
+			parse_config_file(tmp_configlist, defaultfile);
+			main_v->props.cmenu_insert = arraylist_load_new_identifiers_from_list(main_v->props.cmenu_insert, default_insert, 1);
+			main_v->props.cmenu_replace = arraylist_load_new_identifiers_from_list(main_v->props.cmenu_replace, default_replace, 1);
+			main_v->props.lasttime_cust_menu = TIME_T_TO_GINT(time(NULL));
+			free_arraylist(default_replace);
+			free_arraylist(default_insert);
+			free_configlist(tmp_configlist);
+		}
+	}
+	/* for backwards compatibility with older (before Bluefish 0.10) custom menu files we can convert those.. 
+	we will not need the 'type' anymore, since we will put them in separate lists, hence the memmove() call
+	*/
+	if (cust_menu) {
+		GList *tmplist= g_list_first(cust_menu);
+		while (tmplist) {
+			gchar **strarr = (gchar **)tmplist->data;
+			gint count = count_array(strarr);
+			if (count >= 5 && strarr[1][0] == '0') {
+				DEBUG_MSG("rcfile_parse_custom_menu, converting insert, 0=%s, 1=%s\n", strarr[0], strarr[1]);
+				g_free(strarr[1]);
+				memmove(&strarr[1], &strarr[2], (count-1) * sizeof(gchar *));
+				main_v->props.cmenu_insert = g_list_append(main_v->props.cmenu_insert, strarr);
+			} else if (count >= 8 && strarr[1][0] == '1') {
+				DEBUG_MSG("rcfile_parse_custom_menu, converting replace, 0=%s, 1=%s\n", strarr[0], strarr[1]);
+				g_free(strarr[1]);
+				memmove(&strarr[1], &strarr[2], (count-1) * sizeof(gchar *));
+				main_v->props.cmenu_replace = g_list_append(main_v->props.cmenu_replace, strarr);
+			} else if (count >= 4 && count == (4+atoi(strarr[1]))) { /*  the first check avoids a segfault if count == 1 */
+				/* a very old insert type, 0=menupath, 1=numvariables, 2=string1, 3=string2, 4... are variables 
+				   we can re-arrange it for the new insert type */
+				gchar *numvars = strarr[1];
+				strarr[1] = strarr[2];
+				strarr[2] = strarr[3];
+				strarr[3] = numvars; /* the variables; beyond [3], are still the same */
+				DEBUG_MSG("rcfile_parse_custom_menu, converting very old insert, 0=%s\n", strarr[0]);
+				main_v->props.cmenu_insert = g_list_append(main_v->props.cmenu_insert, strarr);
+			} else {
+#ifdef DEBUG
+				if (count > 2) {
+					g_print("rcfile_parse_custom_menu, ignoring %s with type %s (count=%d)\n",strarr[0], strarr[1], count);
+				} else {
+					g_print("rcfile_parse_custom_menu, ignoring invalid cust_menu entry with count=%d..\n", count);
+				}
+#endif
+			}
+			tmplist = g_list_next(tmplist);
+		}
+		g_list_free(cust_menu);
+	}
+	
+/*		main_v->props.cust_menu = g_list_append(main_v->props.cust_menu, array_from_arglist(N_("/php/ibase/ibase fetch row"), "0", "while ($%1 = ibase_fetch_row($%0)) {\n	echo $%1[0];\n", "\n}\nibase_free_result($%0);\n", "2", _("ibase result variable name"), _("row variable name"), NULL));
 		main_v->props.cust_menu = g_list_append(main_v->props.cust_menu, array_from_arglist(N_("/php/mysql/mysql fetch row"), "0", "while ($%1 = mysql_fetch_row($%0)) {\n	echo $%1[0];\n", "\n}\nmysql_free_result($%0);\n", "2", _("mysql result variable name"), _("row variable name"), NULL));
 		main_v->props.cust_menu = g_list_append(main_v->props.cust_menu, array_from_arglist(N_("/php/mysql/mysql connect"), "0", "$mysql_id = mysql_connect('%0', '%1', '%2');\nmysql_select_db('%3', $mysql_id);\n", "", "4", _("mysql host"), _("mysql username"), _("mysql password"), _("database name"), NULL));
 		main_v->props.cust_menu = g_list_append(main_v->props.cust_menu, array_from_arglist(N_("/php/ibase/ibase connect"), "0", "$ibase_id = ibase_connect('%0:%1', '%2', '%3');\n", "", "4", _("ibase host"), _("ibase database path"), _("ibase username"), _("ibase password"), NULL));
@@ -660,7 +764,10 @@ void rcfile_parse_custom_menu(void) {
 		main_v->props.cust_menu = g_list_append(main_v->props.cust_menu, array_from_arglist(N_("/replace/convert <td> in <th> in selection"), "1", "(<|</)(td)([^>]*)(>)", "\\0th\\2>", "2", "1", "0", "0", NULL));
 		main_v->props.cust_menu = g_list_append(main_v->props.cust_menu, array_from_arglist(N_("/replace/convert <any tag> in <any other tag> in selection"), "1", "(<|</)(%0)([^>]*)(>)", "\\0%1\\2>", "2", "1", "0", "2", "any tag name", "any other tag name", NULL));
 	}
+	*/
+
 	g_free(filename);
+	g_free(defaultfile);
 }
 static gint rcfile_save_custom_menu(void) {
 	gint retval;
